import typing

from gi.repository import GObject
from typing_extensions import Self

T = typing.TypeVar("T")

class Connection(GObject.GPointer): ...
class Error(GObject.GPointer): ...
class Message(GObject.GPointer): ...
class MessageIter(GObject.GPointer): ...
class PendingCall(GObject.GPointer): ...

class BusType(GObject.GEnum):
    SESSION = 0
    STARTER = 2
    SYSTEM = 1



import typing

from gi.repository import GObject

T = typing.TypeVar("T")

PI: float = 3.141593
PI_2: float = 1.570796
VEC2_LEN: int = 2
VEC3_LEN: int = 3
VEC4_LEN: int = 4
_lock = ...  # FIXME Constant
_namespace: str = "Graphene"
_version: str = "1.0"

def box_empty() -> Box: ...
def box_infinite() -> Box: ...
def box_minus_one() -> Box: ...
def box_one() -> Box: ...
def box_one_minus_one() -> Box: ...
def box_zero() -> Box: ...
def point3d_zero() -> Point3D: ...
def point_zero() -> Point: ...
def rect_alloc() -> Rect: ...
def rect_zero() -> Rect: ...
def size_zero() -> Size: ...
def vec2_one() -> Vec2: ...
def vec2_x_axis() -> Vec2: ...
def vec2_y_axis() -> Vec2: ...
def vec2_zero() -> Vec2: ...
def vec3_one() -> Vec3: ...
def vec3_x_axis() -> Vec3: ...
def vec3_y_axis() -> Vec3: ...
def vec3_z_axis() -> Vec3: ...
def vec3_zero() -> Vec3: ...
def vec4_one() -> Vec4: ...
def vec4_w_axis() -> Vec4: ...
def vec4_x_axis() -> Vec4: ...
def vec4_y_axis() -> Vec4: ...
def vec4_z_axis() -> Vec4: ...
def vec4_zero() -> Vec4: ...

class Box(GObject.GBoxed):
    """
    :Constructors:

    ::

        Box()
        alloc() -> Graphene.Box
    """

    min: Vec3 = ...
    max: Vec3 = ...
    @classmethod
    def alloc(cls) -> Box: ...
    def contains_box(self, b: Box) -> bool: ...
    def contains_point(self, point: Point3D) -> bool: ...
    @staticmethod
    def empty() -> Box: ...
    def equal(self, b: Box) -> bool: ...
    def expand(self, point: Point3D) -> Box: ...
    def expand_scalar(self, scalar: float) -> Box: ...
    def expand_vec3(self, vec: Vec3) -> Box: ...
    def free(self) -> None: ...
    def get_bounding_sphere(self) -> Sphere: ...
    def get_center(self) -> Point3D: ...
    def get_depth(self) -> float: ...
    def get_height(self) -> float: ...
    def get_max(self) -> Point3D: ...
    def get_min(self) -> Point3D: ...
    def get_size(self) -> Vec3: ...
    def get_vertices(self) -> list[Vec3]: ...
    def get_width(self) -> float: ...
    @staticmethod
    def infinite() -> Box: ...
    def init(
        self, min: typing.Optional[Point3D] = None, max: typing.Optional[Point3D] = None
    ) -> Box: ...
    def init_from_box(self, src: Box) -> Box: ...
    def init_from_points(self, points: typing.Sequence[Point3D]) -> Box: ...
    def init_from_vec3(
        self, min: typing.Optional[Vec3] = None, max: typing.Optional[Vec3] = None
    ) -> Box: ...
    def init_from_vectors(self, vectors: typing.Sequence[Vec3]) -> Box: ...
    def intersection(self, b: Box) -> typing.Tuple[bool, Box]: ...
    @staticmethod
    def minus_one() -> Box: ...
    @staticmethod
    def one() -> Box: ...
    @staticmethod
    def one_minus_one() -> Box: ...
    def union(self, b: Box) -> Box: ...
    @staticmethod
    def zero() -> Box: ...

class Euler(GObject.GBoxed):
    """
    :Constructors:

    ::

        Euler()
        alloc() -> Graphene.Euler
    """

    angles: Vec3 = ...
    order: EulerOrder = ...
    @classmethod
    def alloc(cls) -> Euler: ...
    def equal(self, b: Euler) -> bool: ...
    def free(self) -> None: ...
    def get_alpha(self) -> float: ...
    def get_beta(self) -> float: ...
    def get_gamma(self) -> float: ...
    def get_order(self) -> EulerOrder: ...
    def get_x(self) -> float: ...
    def get_y(self) -> float: ...
    def get_z(self) -> float: ...
    def init(self, x: float, y: float, z: float) -> Euler: ...
    def init_from_euler(self, src: typing.Optional[Euler] = None) -> Euler: ...
    def init_from_matrix(
        self, m: typing.Optional[Matrix], order: EulerOrder
    ) -> Euler: ...
    def init_from_quaternion(
        self, q: typing.Optional[Quaternion], order: EulerOrder
    ) -> Euler: ...
    def init_from_radians(
        self, x: float, y: float, z: float, order: EulerOrder
    ) -> Euler: ...
    def init_from_vec3(self, v: typing.Optional[Vec3], order: EulerOrder) -> Euler: ...
    def init_with_order(
        self, x: float, y: float, z: float, order: EulerOrder
    ) -> Euler: ...
    def reorder(self, order: EulerOrder) -> Euler: ...
    def to_matrix(self) -> Matrix: ...
    def to_quaternion(self) -> Quaternion: ...
    def to_vec3(self) -> Vec3: ...

class Frustum(GObject.GBoxed):
    """
    :Constructors:

    ::

        Frustum()
        alloc() -> Graphene.Frustum
    """

    planes: list[Plane] = ...
    @classmethod
    def alloc(cls) -> Frustum: ...
    def contains_point(self, point: Point3D) -> bool: ...
    def equal(self, b: Frustum) -> bool: ...
    def free(self) -> None: ...
    def get_planes(self) -> list[Plane]: ...
    def init(
        self, p0: Plane, p1: Plane, p2: Plane, p3: Plane, p4: Plane, p5: Plane
    ) -> Frustum: ...
    def init_from_frustum(self, src: Frustum) -> Frustum: ...
    def init_from_matrix(self, matrix: Matrix) -> Frustum: ...
    def intersects_box(self, box: Box) -> bool: ...
    def intersects_sphere(self, sphere: Sphere) -> bool: ...

class Matrix(GObject.GBoxed):
    """
    :Constructors:

    ::

        Matrix()
        alloc() -> Graphene.Matrix
    """

    value: Simd4X4F = ...
    @classmethod
    def alloc(cls) -> Matrix: ...
    def decompose(self) -> typing.Tuple[bool, Vec3, Vec3, Quaternion, Vec3, Vec4]: ...
    def determinant(self) -> float: ...
    def equal(self, b: Matrix) -> bool: ...
    def equal_fast(self, b: Matrix) -> bool: ...
    def free(self) -> None: ...
    def get_row(self, index_: int) -> Vec4: ...
    def get_value(self, row: int, col: int) -> float: ...
    def get_x_scale(self) -> float: ...
    def get_x_translation(self) -> float: ...
    def get_y_scale(self) -> float: ...
    def get_y_translation(self) -> float: ...
    def get_z_scale(self) -> float: ...
    def get_z_translation(self) -> float: ...
    def init_from_2d(
        self, xx: float, yx: float, xy: float, yy: float, x_0: float, y_0: float
    ) -> Matrix: ...
    def init_from_float(self, v: typing.Sequence[float]) -> Matrix: ...
    def init_from_matrix(self, src: Matrix) -> Matrix: ...
    def init_from_vec4(self, v0: Vec4, v1: Vec4, v2: Vec4, v3: Vec4) -> Matrix: ...
    def init_frustum(
        self,
        left: float,
        right: float,
        bottom: float,
        top: float,
        z_near: float,
        z_far: float,
    ) -> Matrix: ...
    def init_identity(self) -> Matrix: ...
    def init_look_at(self, eye: Vec3, center: Vec3, up: Vec3) -> Matrix: ...
    def init_ortho(
        self,
        left: float,
        right: float,
        top: float,
        bottom: float,
        z_near: float,
        z_far: float,
    ) -> Matrix: ...
    def init_perspective(
        self, fovy: float, aspect: float, z_near: float, z_far: float
    ) -> Matrix: ...
    def init_rotate(self, angle: float, axis: Vec3) -> Matrix: ...
    def init_scale(self, x: float, y: float, z: float) -> Matrix: ...
    def init_skew(self, x_skew: float, y_skew: float) -> Matrix: ...
    def init_translate(self, p: Point3D) -> Matrix: ...
    def interpolate(self, b: Matrix, factor: float) -> Matrix: ...
    def inverse(self) -> typing.Tuple[bool, Matrix]: ...
    def is_2d(self) -> bool: ...
    def is_backface_visible(self) -> bool: ...
    def is_identity(self) -> bool: ...
    def is_singular(self) -> bool: ...
    def multiply(self, b: Matrix) -> Matrix: ...
    def near(self, b: Matrix, epsilon: float) -> bool: ...
    def normalize(self) -> Matrix: ...
    def perspective(self, depth: float) -> Matrix: ...
    def print_(self) -> None: ...
    def project_point(self, p: Point) -> Point: ...
    def project_rect(self, r: Rect) -> Quad: ...
    def project_rect_bounds(self, r: Rect) -> Rect: ...
    def rotate(self, angle: float, axis: Vec3) -> None: ...
    def rotate_euler(self, e: Euler) -> None: ...
    def rotate_quaternion(self, q: Quaternion) -> None: ...
    def rotate_x(self, angle: float) -> None: ...
    def rotate_y(self, angle: float) -> None: ...
    def rotate_z(self, angle: float) -> None: ...
    def scale(self, factor_x: float, factor_y: float, factor_z: float) -> None: ...
    def skew_xy(self, factor: float) -> None: ...
    def skew_xz(self, factor: float) -> None: ...
    def skew_yz(self, factor: float) -> None: ...
    def to_2d(self) -> typing.Tuple[bool, float, float, float, float, float, float]: ...
    def to_float(self) -> list[float]: ...
    def transform_bounds(self, r: Rect) -> Rect: ...
    def transform_box(self, b: Box) -> Box: ...
    def transform_point(self, p: Point) -> Point: ...
    def transform_point3d(self, p: Point3D) -> Point3D: ...
    def transform_ray(self, r: Ray) -> Ray: ...
    def transform_rect(self, r: Rect) -> Quad: ...
    def transform_sphere(self, s: Sphere) -> Sphere: ...
    def transform_vec3(self, v: Vec3) -> Vec3: ...
    def transform_vec4(self, v: Vec4) -> Vec4: ...
    def translate(self, pos: Point3D) -> None: ...
    def transpose(self) -> Matrix: ...
    def unproject_point3d(self, modelview: Matrix, point: Point3D) -> Point3D: ...
    def untransform_bounds(self, r: Rect, bounds: Rect) -> Rect: ...
    def untransform_point(
        self, p: Point, bounds: Rect
    ) -> typing.Tuple[bool, Point]: ...

class Plane(GObject.GBoxed):
    """
    :Constructors:

    ::

        Plane()
        alloc() -> Graphene.Plane
    """

    normal: Vec3 = ...
    constant: float = ...
    @classmethod
    def alloc(cls) -> Plane: ...
    def distance(self, point: Point3D) -> float: ...
    def equal(self, b: Plane) -> bool: ...
    def free(self) -> None: ...
    def get_constant(self) -> float: ...
    def get_normal(self) -> Vec3: ...
    def init(self, normal: typing.Optional[Vec3], constant: float) -> Plane: ...
    def init_from_plane(self, src: Plane) -> Plane: ...
    def init_from_point(self, normal: Vec3, point: Point3D) -> Plane: ...
    def init_from_points(self, a: Point3D, b: Point3D, c: Point3D) -> Plane: ...
    def init_from_vec4(self, src: Vec4) -> Plane: ...
    def negate(self) -> Plane: ...
    def normalize(self) -> Plane: ...
    def transform(
        self, matrix: Matrix, normal_matrix: typing.Optional[Matrix] = None
    ) -> Plane: ...

class Point(GObject.GBoxed):
    """
    :Constructors:

    ::

        Point()
        alloc() -> Graphene.Point
    """

    x: float = ...
    y: float = ...
    @classmethod
    def alloc(cls) -> Point: ...
    def distance(self, b: Point) -> typing.Tuple[float, float, float]: ...
    def equal(self, b: Point) -> bool: ...
    def free(self) -> None: ...
    def init(self, x: float, y: float) -> Point: ...
    def init_from_point(self, src: Point) -> Point: ...
    def init_from_vec2(self, src: Vec2) -> Point: ...
    def interpolate(self, b: Point, factor: float) -> Point: ...
    def near(self, b: Point, epsilon: float) -> bool: ...
    def to_vec2(self) -> Vec2: ...
    @staticmethod
    def zero() -> Point: ...

class Point3D(GObject.GBoxed):
    """
    :Constructors:

    ::

        Point3D()
        alloc() -> Graphene.Point3D
    """

    x: float = ...
    y: float = ...
    z: float = ...
    @classmethod
    def alloc(cls) -> Point3D: ...
    def cross(self, b: Point3D) -> Point3D: ...
    def distance(self, b: Point3D) -> typing.Tuple[float, Vec3]: ...
    def dot(self, b: Point3D) -> float: ...
    def equal(self, b: Point3D) -> bool: ...
    def free(self) -> None: ...
    def init(self, x: float, y: float, z: float) -> Point3D: ...
    def init_from_point(self, src: Point3D) -> Point3D: ...
    def init_from_vec3(self, v: Vec3) -> Point3D: ...
    def interpolate(self, b: Point3D, factor: float) -> Point3D: ...
    def length(self) -> float: ...
    def near(self, b: Point3D, epsilon: float) -> bool: ...
    def normalize(self) -> Point3D: ...
    def normalize_viewport(
        self, viewport: Rect, z_near: float, z_far: float
    ) -> Point3D: ...
    def scale(self, factor: float) -> Point3D: ...
    def to_vec3(self) -> Vec3: ...
    @staticmethod
    def zero() -> Point3D: ...

class Quad(GObject.GBoxed):
    """
    :Constructors:

    ::

        Quad()
        alloc() -> Graphene.Quad
    """

    points: list[Point] = ...
    @classmethod
    def alloc(cls) -> Quad: ...
    def bounds(self) -> Rect: ...
    def contains(self, p: Point) -> bool: ...
    def free(self) -> None: ...
    def get_point(self, index_: int) -> Point: ...
    def init(self, p1: Point, p2: Point, p3: Point, p4: Point) -> Quad: ...
    def init_from_points(self, points: typing.Sequence[Point]) -> Quad: ...
    def init_from_rect(self, r: Rect) -> Quad: ...

class Quaternion(GObject.GBoxed):
    """
    :Constructors:

    ::

        Quaternion()
        alloc() -> Graphene.Quaternion
    """

    x: float = ...
    y: float = ...
    z: float = ...
    w: float = ...
    def add(self, b: Quaternion) -> Quaternion: ...
    @classmethod
    def alloc(cls) -> Quaternion: ...
    def dot(self, b: Quaternion) -> float: ...
    def equal(self, b: Quaternion) -> bool: ...
    def free(self) -> None: ...
    def init(self, x: float, y: float, z: float, w: float) -> Quaternion: ...
    def init_from_angle_vec3(self, angle: float, axis: Vec3) -> Quaternion: ...
    def init_from_angles(
        self, deg_x: float, deg_y: float, deg_z: float
    ) -> Quaternion: ...
    def init_from_euler(self, e: Euler) -> Quaternion: ...
    def init_from_matrix(self, m: Matrix) -> Quaternion: ...
    def init_from_quaternion(self, src: Quaternion) -> Quaternion: ...
    def init_from_radians(
        self, rad_x: float, rad_y: float, rad_z: float
    ) -> Quaternion: ...
    def init_from_vec4(self, src: Vec4) -> Quaternion: ...
    def init_identity(self) -> Quaternion: ...
    def invert(self) -> Quaternion: ...
    def multiply(self, b: Quaternion) -> Quaternion: ...
    def normalize(self) -> Quaternion: ...
    def scale(self, factor: float) -> Quaternion: ...
    def slerp(self, b: Quaternion, factor: float) -> Quaternion: ...
    def to_angle_vec3(self) -> typing.Tuple[float, Vec3]: ...
    def to_angles(self) -> typing.Tuple[float, float, float]: ...
    def to_matrix(self) -> Matrix: ...
    def to_radians(self) -> typing.Tuple[float, float, float]: ...
    def to_vec4(self) -> Vec4: ...

class Ray(GObject.GBoxed):
    """
    :Constructors:

    ::

        Ray()
        alloc() -> Graphene.Ray
    """

    origin: Vec3 = ...
    direction: Vec3 = ...
    @classmethod
    def alloc(cls) -> Ray: ...
    def equal(self, b: Ray) -> bool: ...
    def free(self) -> None: ...
    def get_closest_point_to_point(self, p: Point3D) -> Point3D: ...
    def get_direction(self) -> Vec3: ...
    def get_distance_to_plane(self, p: Plane) -> float: ...
    def get_distance_to_point(self, p: Point3D) -> float: ...
    def get_origin(self) -> Point3D: ...
    def get_position_at(self, t: float) -> Point3D: ...
    def init(
        self,
        origin: typing.Optional[Point3D] = None,
        direction: typing.Optional[Vec3] = None,
    ) -> Ray: ...
    def init_from_ray(self, src: Ray) -> Ray: ...
    def init_from_vec3(
        self,
        origin: typing.Optional[Vec3] = None,
        direction: typing.Optional[Vec3] = None,
    ) -> Ray: ...
    def intersect_box(self, b: Box) -> typing.Tuple[RayIntersectionKind, float]: ...
    def intersect_sphere(
        self, s: Sphere
    ) -> typing.Tuple[RayIntersectionKind, float]: ...
    def intersect_triangle(
        self, t: Triangle
    ) -> typing.Tuple[RayIntersectionKind, float]: ...
    def intersects_box(self, b: Box) -> bool: ...
    def intersects_sphere(self, s: Sphere) -> bool: ...
    def intersects_triangle(self, t: Triangle) -> bool: ...

class Rect(GObject.GBoxed):
    """
    :Constructors:

    ::

        Rect()
    """

    origin: Point = ...
    size: Size = ...
    @staticmethod
    def alloc() -> Rect: ...
    def contains_point(self, p: Point) -> bool: ...
    def contains_rect(self, b: Rect) -> bool: ...
    def equal(self, b: Rect) -> bool: ...
    def expand(self, p: Point) -> Rect: ...
    def free(self) -> None: ...
    def get_area(self) -> float: ...
    def get_bottom_left(self) -> Point: ...
    def get_bottom_right(self) -> Point: ...
    def get_center(self) -> Point: ...
    def get_height(self) -> float: ...
    def get_top_left(self) -> Point: ...
    def get_top_right(self) -> Point: ...
    def get_vertices(self) -> list[Vec2]: ...
    def get_width(self) -> float: ...
    def get_x(self) -> float: ...
    def get_y(self) -> float: ...
    def init(self, x: float, y: float, width: float, height: float) -> Rect: ...
    def init_from_rect(self, src: Rect) -> Rect: ...
    def inset(self, d_x: float, d_y: float) -> Rect: ...
    def inset_r(self, d_x: float, d_y: float) -> Rect: ...
    def interpolate(self, b: Rect, factor: float) -> Rect: ...
    def intersection(self, b: Rect) -> typing.Tuple[bool, Rect]: ...
    def normalize(self) -> Rect: ...
    def normalize_r(self) -> Rect: ...
    def offset(self, d_x: float, d_y: float) -> Rect: ...
    def offset_r(self, d_x: float, d_y: float) -> Rect: ...
    def round(self) -> Rect: ...
    def round_extents(self) -> Rect: ...
    def round_to_pixel(self) -> Rect: ...
    def scale(self, s_h: float, s_v: float) -> Rect: ...
    def union(self, b: Rect) -> Rect: ...
    @staticmethod
    def zero() -> Rect: ...

class Simd4F(GObject.GPointer):
    """
    :Constructors:

    ::

        Simd4F()
    """

    x: float = ...
    y: float = ...
    z: float = ...
    w: float = ...

class Simd4X4F(GObject.GPointer):
    """
    :Constructors:

    ::

        Simd4X4F()
    """

    x: Simd4F = ...
    y: Simd4F = ...
    z: Simd4F = ...
    w: Simd4F = ...

class Size(GObject.GBoxed):
    """
    :Constructors:

    ::

        Size()
        alloc() -> Graphene.Size
    """

    width: float = ...
    height: float = ...
    @classmethod
    def alloc(cls) -> Size: ...
    def equal(self, b: Size) -> bool: ...
    def free(self) -> None: ...
    def init(self, width: float, height: float) -> Size: ...
    def init_from_size(self, src: Size) -> Size: ...
    def interpolate(self, b: Size, factor: float) -> Size: ...
    def scale(self, factor: float) -> Size: ...
    @staticmethod
    def zero() -> Size: ...

class Sphere(GObject.GBoxed):
    """
    :Constructors:

    ::

        Sphere()
        alloc() -> Graphene.Sphere
    """

    center: Vec3 = ...
    radius: float = ...
    @classmethod
    def alloc(cls) -> Sphere: ...
    def contains_point(self, point: Point3D) -> bool: ...
    def distance(self, point: Point3D) -> float: ...
    def equal(self, b: Sphere) -> bool: ...
    def free(self) -> None: ...
    def get_bounding_box(self) -> Box: ...
    def get_center(self) -> Point3D: ...
    def get_radius(self) -> float: ...
    def init(self, center: typing.Optional[Point3D], radius: float) -> Sphere: ...
    def init_from_points(
        self, points: typing.Sequence[Point3D], center: typing.Optional[Point3D] = None
    ) -> Sphere: ...
    def init_from_vectors(
        self, vectors: typing.Sequence[Vec3], center: typing.Optional[Point3D] = None
    ) -> Sphere: ...
    def is_empty(self) -> bool: ...
    def translate(self, point: Point3D) -> Sphere: ...

class Triangle(GObject.GBoxed):
    """
    :Constructors:

    ::

        Triangle()
        alloc() -> Graphene.Triangle
    """

    a: Vec3 = ...
    b: Vec3 = ...
    c: Vec3 = ...
    @classmethod
    def alloc(cls) -> Triangle: ...
    def contains_point(self, p: Point3D) -> bool: ...
    def equal(self, b: Triangle) -> bool: ...
    def free(self) -> None: ...
    def get_area(self) -> float: ...
    def get_barycoords(
        self, p: typing.Optional[Point3D] = None
    ) -> typing.Tuple[bool, Vec2]: ...
    def get_bounding_box(self) -> Box: ...
    def get_midpoint(self) -> Point3D: ...
    def get_normal(self) -> Vec3: ...
    def get_plane(self) -> Plane: ...
    def get_points(self) -> typing.Tuple[Point3D, Point3D, Point3D]: ...
    def get_uv(
        self, p: typing.Optional[Point3D], uv_a: Vec2, uv_b: Vec2, uv_c: Vec2
    ) -> typing.Tuple[bool, Vec2]: ...
    def get_vertices(self) -> typing.Tuple[Vec3, Vec3, Vec3]: ...
    def init_from_float(
        self,
        a: typing.Sequence[float],
        b: typing.Sequence[float],
        c: typing.Sequence[float],
    ) -> Triangle: ...
    def init_from_point3d(
        self,
        a: typing.Optional[Point3D] = None,
        b: typing.Optional[Point3D] = None,
        c: typing.Optional[Point3D] = None,
    ) -> Triangle: ...
    def init_from_vec3(
        self,
        a: typing.Optional[Vec3] = None,
        b: typing.Optional[Vec3] = None,
        c: typing.Optional[Vec3] = None,
    ) -> Triangle: ...

class Vec2(GObject.GBoxed):
    """
    :Constructors:

    ::

        Vec2()
        alloc() -> Graphene.Vec2
    """

    value: Simd4F = ...
    def add(self, b: Vec2) -> Vec2: ...
    @classmethod
    def alloc(cls) -> Vec2: ...
    def divide(self, b: Vec2) -> Vec2: ...
    def dot(self, b: Vec2) -> float: ...
    def equal(self, v2: Vec2) -> bool: ...
    def free(self) -> None: ...
    def get_x(self) -> float: ...
    def get_y(self) -> float: ...
    def init(self, x: float, y: float) -> Vec2: ...
    def init_from_float(self, src: typing.Sequence[float]) -> Vec2: ...
    def init_from_vec2(self, src: Vec2) -> Vec2: ...
    def interpolate(self, v2: Vec2, factor: float) -> Vec2: ...
    def length(self) -> float: ...
    def max(self, b: Vec2) -> Vec2: ...
    def min(self, b: Vec2) -> Vec2: ...
    def multiply(self, b: Vec2) -> Vec2: ...
    def near(self, v2: Vec2, epsilon: float) -> bool: ...
    def negate(self) -> Vec2: ...
    def normalize(self) -> Vec2: ...
    @staticmethod
    def one() -> Vec2: ...
    def scale(self, factor: float) -> Vec2: ...
    def subtract(self, b: Vec2) -> Vec2: ...
    def to_float(self) -> list[float]: ...
    @staticmethod
    def x_axis() -> Vec2: ...
    @staticmethod
    def y_axis() -> Vec2: ...
    @staticmethod
    def zero() -> Vec2: ...

class Vec3(GObject.GBoxed):
    """
    :Constructors:

    ::

        Vec3()
        alloc() -> Graphene.Vec3
    """

    value: Simd4F = ...
    def add(self, b: Vec3) -> Vec3: ...
    @classmethod
    def alloc(cls) -> Vec3: ...
    def cross(self, b: Vec3) -> Vec3: ...
    def divide(self, b: Vec3) -> Vec3: ...
    def dot(self, b: Vec3) -> float: ...
    def equal(self, v2: Vec3) -> bool: ...
    def free(self) -> None: ...
    def get_x(self) -> float: ...
    def get_xy(self) -> Vec2: ...
    def get_xy0(self) -> Vec3: ...
    def get_xyz0(self) -> Vec4: ...
    def get_xyz1(self) -> Vec4: ...
    def get_xyzw(self, w: float) -> Vec4: ...
    def get_y(self) -> float: ...
    def get_z(self) -> float: ...
    def init(self, x: float, y: float, z: float) -> Vec3: ...
    def init_from_float(self, src: typing.Sequence[float]) -> Vec3: ...
    def init_from_vec3(self, src: Vec3) -> Vec3: ...
    def interpolate(self, v2: Vec3, factor: float) -> Vec3: ...
    def length(self) -> float: ...
    def max(self, b: Vec3) -> Vec3: ...
    def min(self, b: Vec3) -> Vec3: ...
    def multiply(self, b: Vec3) -> Vec3: ...
    def near(self, v2: Vec3, epsilon: float) -> bool: ...
    def negate(self) -> Vec3: ...
    def normalize(self) -> Vec3: ...
    @staticmethod
    def one() -> Vec3: ...
    def scale(self, factor: float) -> Vec3: ...
    def subtract(self, b: Vec3) -> Vec3: ...
    def to_float(self) -> list[float]: ...
    @staticmethod
    def x_axis() -> Vec3: ...
    @staticmethod
    def y_axis() -> Vec3: ...
    @staticmethod
    def z_axis() -> Vec3: ...
    @staticmethod
    def zero() -> Vec3: ...

class Vec4(GObject.GBoxed):
    """
    :Constructors:

    ::

        Vec4()
        alloc() -> Graphene.Vec4
    """

    value: Simd4F = ...
    def add(self, b: Vec4) -> Vec4: ...
    @classmethod
    def alloc(cls) -> Vec4: ...
    def divide(self, b: Vec4) -> Vec4: ...
    def dot(self, b: Vec4) -> float: ...
    def equal(self, v2: Vec4) -> bool: ...
    def free(self) -> None: ...
    def get_w(self) -> float: ...
    def get_x(self) -> float: ...
    def get_xy(self) -> Vec2: ...
    def get_xyz(self) -> Vec3: ...
    def get_y(self) -> float: ...
    def get_z(self) -> float: ...
    def init(self, x: float, y: float, z: float, w: float) -> Vec4: ...
    def init_from_float(self, src: typing.Sequence[float]) -> Vec4: ...
    def init_from_vec2(self, src: Vec2, z: float, w: float) -> Vec4: ...
    def init_from_vec3(self, src: Vec3, w: float) -> Vec4: ...
    def init_from_vec4(self, src: Vec4) -> Vec4: ...
    def interpolate(self, v2: Vec4, factor: float) -> Vec4: ...
    def length(self) -> float: ...
    def max(self, b: Vec4) -> Vec4: ...
    def min(self, b: Vec4) -> Vec4: ...
    def multiply(self, b: Vec4) -> Vec4: ...
    def near(self, v2: Vec4, epsilon: float) -> bool: ...
    def negate(self) -> Vec4: ...
    def normalize(self) -> Vec4: ...
    @staticmethod
    def one() -> Vec4: ...
    def scale(self, factor: float) -> Vec4: ...
    def subtract(self, b: Vec4) -> Vec4: ...
    def to_float(self) -> list[float]: ...
    @staticmethod
    def w_axis() -> Vec4: ...
    @staticmethod
    def x_axis() -> Vec4: ...
    @staticmethod
    def y_axis() -> Vec4: ...
    @staticmethod
    def z_axis() -> Vec4: ...
    @staticmethod
    def zero() -> Vec4: ...

class EulerOrder(GObject.GEnum):
    DEFAULT = -1
    RXYX = 19
    RXYZ = 28
    RXZX = 21
    RXZY = 22
    RYXY = 25
    RYXZ = 26
    RYZX = 20
    RYZY = 23
    RZXY = 24
    RZXZ = 27
    RZYX = 18
    RZYZ = 29
    SXYX = 7
    SXYZ = 6
    SXZX = 9
    SXZY = 8
    SYXY = 13
    SYXZ = 12
    SYZX = 10
    SYZY = 11
    SZXY = 14
    SZXZ = 15
    SZYX = 16
    SZYZ = 17
    XYZ = 0
    XZY = 3
    YXZ = 4
    YZX = 1
    ZXY = 2
    ZYX = 5

class RayIntersectionKind(GObject.GEnum):
    ENTER = 1
    LEAVE = 2
    NONE = 0



import typing

T = typing.TypeVar("T")

_lock = ...  # FIXME Constant
_namespace: str = "GLibWin32"
_version: str = "2.0"

def check_windows_version(
    major: int, minor: int, spver: int, os_type: OSType
) -> bool: ...
def error_message(error: int) -> str: ...
def ftruncate(f: int, size: int) -> int: ...
def get_package_installation_directory(package: str, dll_name: str) -> str: ...
def get_package_installation_directory_of_module(hmodule: None) -> str: ...
def get_package_installation_subdirectory(
    package: str, dll_name: str, subdir: str
) -> str: ...
def get_windows_version() -> int: ...
def getlocale() -> str: ...
def locale_filename_from_utf8(utf8filename: str) -> str: ...

class OSType:
    ANY: OSType = 0
    SERVER: OSType = 2
    WORKSTATION: OSType = 1
    denominator = ...  # FIXME Constant
    imag = ...  # FIXME Constant
    numerator = ...  # FIXME Constant
    real = ...  # FIXME Constant

    def as_integer_ratio(self, /): ...  # FIXME Function
    def bit_count(self, /): ...  # FIXME Function
    def bit_length(self, /): ...  # FIXME Function
    def conjugate(self, *args, **kwargs): ...  # FIXME Function
    def from_bytes(bytes, byteorder="big", *, signed=False): ...  # FIXME Function
    def is_integer(self, /): ...  # FIXME Function
    def to_bytes(
        self, /, length=1, byteorder="big", *, signed=False
    ): ...  # FIXME Function



import typing

from gi.repository import Gio
from gi.repository import GLib
from gi.repository import GModule
from gi.repository import GObject
from typing_extensions import Self

T = typing.TypeVar("T")

PIXBUF_MAJOR: int = 2
PIXBUF_MICRO: int = 12
PIXBUF_MINOR: int = 42
PIXBUF_VERSION: str = "2.42.12"
_lock = ...  # FIXME Constant
_namespace: str = "GdkPixbuf"
_overrides_module = ...  # FIXME Constant
_version: str = "2.0"

def pixbuf_error_quark() -> int: ...

class Pixbuf(GObject.Object, Gio.Icon, Gio.LoadableIcon):
    """
    :Constructors:

    ::

        Pixbuf(**properties)
        new(colorspace:GdkPixbuf.Colorspace, has_alpha:bool, bits_per_sample:int, width:int, height:int) -> GdkPixbuf.Pixbuf or None
        new_from_bytes(data:GLib.Bytes, colorspace:GdkPixbuf.Colorspace, has_alpha:bool, bits_per_sample:int, width:int, height:int, rowstride:int) -> GdkPixbuf.Pixbuf
        new_from_data(data:list, colorspace:GdkPixbuf.Colorspace, has_alpha:bool, bits_per_sample:int, width:int, height:int, rowstride:int, destroy_fn:GdkPixbuf.PixbufDestroyNotify=None, destroy_fn_data=None) -> GdkPixbuf.Pixbuf
        new_from_file(filename:str) -> GdkPixbuf.Pixbuf or None
        new_from_file_at_scale(filename:str, width:int, height:int, preserve_aspect_ratio:bool) -> GdkPixbuf.Pixbuf or None
        new_from_file_at_size(filename:str, width:int, height:int) -> GdkPixbuf.Pixbuf or None
        new_from_inline(data:list, copy_pixels:bool) -> GdkPixbuf.Pixbuf
        new_from_resource(resource_path:str) -> GdkPixbuf.Pixbuf or None
        new_from_resource_at_scale(resource_path:str, width:int, height:int, preserve_aspect_ratio:bool) -> GdkPixbuf.Pixbuf or None
        new_from_stream(stream:Gio.InputStream, cancellable:Gio.Cancellable=None) -> GdkPixbuf.Pixbuf or None
        new_from_stream_at_scale(stream:Gio.InputStream, width:int, height:int, preserve_aspect_ratio:bool, cancellable:Gio.Cancellable=None) -> GdkPixbuf.Pixbuf or None
        new_from_stream_finish(async_result:Gio.AsyncResult) -> GdkPixbuf.Pixbuf or None
        new_from_xpm_data(data:list) -> GdkPixbuf.Pixbuf or None

    Object GdkPixbuf

    Properties from GdkPixbuf:
      colorspace -> GdkColorspace: Colorspace
        The colorspace in which the samples are interpreted
      n-channels -> gint: Number of Channels
        The number of samples per pixel
      has-alpha -> gboolean: Has Alpha
        Whether the pixbuf has an alpha channel
      bits-per-sample -> gint: Bits per Sample
        The number of bits per sample
      width -> gint: Width
        The number of columns of the pixbuf
      height -> gint: Height
        The number of rows of the pixbuf
      rowstride -> gint: Rowstride
        The number of bytes between the start of a row and the start of the next row
      pixels -> gpointer: Pixels
        A pointer to the pixel data of the pixbuf
      pixel-bytes -> GBytes: Pixel Bytes
        Readonly pixel data

    Signals from GObject:
      notify (GParam)
    """
    class Props(GObject.Object.Props):
        bits_per_sample: int
        colorspace: Colorspace
        has_alpha: bool
        height: int
        n_channels: int
        pixel_bytes: GLib.Bytes
        pixels: None
        rowstride: int
        width: int

    props: Props = ...
    def __init__(
        self,
        bits_per_sample: int = ...,
        colorspace: Colorspace = ...,
        has_alpha: bool = ...,
        height: int = ...,
        n_channels: int = ...,
        pixel_bytes: GLib.Bytes = ...,
        pixels: None = ...,
        rowstride: int = ...,
        width: int = ...,
    ) -> None: ...
    def add_alpha(
        self, substitute_color: bool, r: int, g: int, b: int
    ) -> typing.Optional[Pixbuf]: ...
    def apply_embedded_orientation(self) -> typing.Optional[Pixbuf]: ...
    @staticmethod
    def calculate_rowstride(
        colorspace: Colorspace,
        has_alpha: bool,
        bits_per_sample: int,
        width: int,
        height: int,
    ) -> int: ...
    def composite(
        self,
        dest: Pixbuf,
        dest_x: int,
        dest_y: int,
        dest_width: int,
        dest_height: int,
        offset_x: float,
        offset_y: float,
        scale_x: float,
        scale_y: float,
        interp_type: InterpType,
        overall_alpha: int,
    ) -> None: ...
    def composite_color(
        self,
        dest: Pixbuf,
        dest_x: int,
        dest_y: int,
        dest_width: int,
        dest_height: int,
        offset_x: float,
        offset_y: float,
        scale_x: float,
        scale_y: float,
        interp_type: InterpType,
        overall_alpha: int,
        check_x: int,
        check_y: int,
        check_size: int,
        color1: int,
        color2: int,
    ) -> None: ...
    def composite_color_simple(
        self,
        dest_width: int,
        dest_height: int,
        interp_type: InterpType,
        overall_alpha: int,
        check_size: int,
        color1: int,
        color2: int,
    ) -> typing.Optional[Pixbuf]: ...
    def copy(self) -> typing.Optional[Pixbuf]: ...
    def copy_area(
        self,
        src_x: int,
        src_y: int,
        width: int,
        height: int,
        dest_pixbuf: Pixbuf,
        dest_x: int,
        dest_y: int,
    ) -> None: ...
    def copy_options(self, dest_pixbuf: Pixbuf) -> bool: ...
    def fill(self, pixel: int) -> None: ...
    def flip(self, horizontal: bool) -> typing.Optional[Pixbuf]: ...
    def get_bits_per_sample(self) -> int: ...
    def get_byte_length(self) -> int: ...
    def get_colorspace(self) -> Colorspace: ...
    @staticmethod
    def get_file_info(
        filename: str,
    ) -> typing.Tuple[typing.Optional[PixbufFormat], int, int]: ...
    @staticmethod
    def get_file_info_async(
        filename: str,
        cancellable: typing.Optional[Gio.Cancellable] = None,
        callback: typing.Optional[typing.Callable[..., None]] = None,
        *user_data: typing.Any,
    ) -> None: ...
    @staticmethod
    def get_file_info_finish(
        async_result: Gio.AsyncResult,
    ) -> typing.Tuple[PixbufFormat, int, int]: ...
    @staticmethod
    def get_formats() -> list[PixbufFormat]: ...
    def get_has_alpha(self) -> bool: ...
    def get_height(self) -> int: ...
    def get_n_channels(self) -> int: ...
    def get_option(self, key: str) -> typing.Optional[str]: ...
    def get_options(self) -> dict[str, str]: ...
    def get_pixels(self) -> bytes: ...
    def get_rowstride(self) -> int: ...
    def get_width(self) -> int: ...
    @staticmethod
    def init_modules(path: str) -> bool: ...
    @classmethod
    def new(
        cls,
        colorspace: Colorspace,
        has_alpha: bool,
        bits_per_sample: int,
        width: int,
        height: int,
    ) -> typing.Optional[Pixbuf]: ...
    @classmethod
    def new_from_bytes(
        cls,
        data: GLib.Bytes,
        colorspace: Colorspace,
        has_alpha: bool,
        bits_per_sample: int,
        width: int,
        height: int,
        rowstride: int,
    ) -> Pixbuf: ...
    # override
    @classmethod
    def new_from_data(
        cls,
        data: GLib.Bytes,
        colorspace: Colorspace,
        has_alpha: bool,
        bits_per_sample: int,
        width: int,
        height: int,
        rowstride: int,
        destroy_fn: typing.Any,
        *destroy_fn_data: typing.Any,
    ) -> Pixbuf: ...
    @classmethod
    def new_from_file(cls, filename: str) -> typing.Optional[Pixbuf]: ...
    @classmethod
    def new_from_file_at_scale(
        cls, filename: str, width: int, height: int, preserve_aspect_ratio: bool
    ) -> typing.Optional[Pixbuf]: ...
    @classmethod
    def new_from_file_at_size(
        cls, filename: str, width: int, height: int
    ) -> typing.Optional[Pixbuf]: ...
    @classmethod
    def new_from_inline(
        cls, data: typing.Sequence[int], copy_pixels: bool
    ) -> Pixbuf: ...
    @classmethod
    def new_from_resource(cls, resource_path: str) -> typing.Optional[Pixbuf]: ...
    @classmethod
    def new_from_resource_at_scale(
        cls, resource_path: str, width: int, height: int, preserve_aspect_ratio: bool
    ) -> typing.Optional[Pixbuf]: ...
    @classmethod
    def new_from_stream(
        cls,
        stream: Gio.InputStream,
        cancellable: typing.Optional[Gio.Cancellable] = None,
    ) -> typing.Optional[Pixbuf]: ...
    @staticmethod
    def new_from_stream_async(
        stream: Gio.InputStream,
        cancellable: typing.Optional[Gio.Cancellable] = None,
        callback: typing.Optional[typing.Callable[..., None]] = None,
        *user_data: typing.Any,
    ) -> None: ...
    @classmethod
    def new_from_stream_at_scale(
        cls,
        stream: Gio.InputStream,
        width: int,
        height: int,
        preserve_aspect_ratio: bool,
        cancellable: typing.Optional[Gio.Cancellable] = None,
    ) -> typing.Optional[Pixbuf]: ...
    @staticmethod
    def new_from_stream_at_scale_async(
        stream: Gio.InputStream,
        width: int,
        height: int,
        preserve_aspect_ratio: bool,
        cancellable: typing.Optional[Gio.Cancellable] = None,
        callback: typing.Optional[typing.Callable[..., None]] = None,
        *user_data: typing.Any,
    ) -> None: ...
    @classmethod
    def new_from_stream_finish(cls, async_result: Gio.AsyncResult) -> Pixbuf: ...
    @classmethod
    def new_from_xpm_data(
        cls, data: typing.Sequence[str]
    ) -> typing.Optional[Pixbuf]: ...
    def new_subpixbuf(
        self, src_x: int, src_y: int, width: int, height: int
    ) -> Pixbuf: ...
    def read_pixel_bytes(self) -> GLib.Bytes: ...
    def read_pixels(self) -> int: ...
    def remove_option(self, key: str) -> bool: ...
    def rotate_simple(self, angle: PixbufRotation) -> typing.Optional[Pixbuf]: ...
    def saturate_and_pixelate(
        self, dest: Pixbuf, saturation: float, pixelate: bool
    ) -> None: ...
    def save_to_bufferv(
        self,
        type: str,
        option_keys: typing.Optional[typing.Sequence[str]] = None,
        option_values: typing.Optional[typing.Sequence[str]] = None,
    ) -> typing.Tuple[bool, bytes]: ...
    def save_to_callbackv(
        self,
        save_func: typing.Callable[..., typing.Tuple[bool, GLib.Error]],
        type: str,
        option_keys: typing.Optional[typing.Sequence[str]] = None,
        option_values: typing.Optional[typing.Sequence[str]] = None,
        *user_data: typing.Any,
    ) -> bool: ...
    @staticmethod
    def save_to_stream_finish(async_result: Gio.AsyncResult) -> bool: ...
    def save_to_streamv(
        self,
        stream: Gio.OutputStream,
        type: str,
        option_keys: typing.Optional[typing.Sequence[str]] = None,
        option_values: typing.Optional[typing.Sequence[str]] = None,
        cancellable: typing.Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def save_to_streamv_async(
        self,
        stream: Gio.OutputStream,
        type: str,
        option_keys: typing.Optional[typing.Sequence[str]] = None,
        option_values: typing.Optional[typing.Sequence[str]] = None,
        cancellable: typing.Optional[Gio.Cancellable] = None,
        callback: typing.Optional[typing.Callable[..., None]] = None,
        *user_data: typing.Any,
    ) -> None: ...
    def savev(
        self,
        filename: str,
        type: str,
        option_keys: typing.Optional[typing.Sequence[str]] = None,
        option_values: typing.Optional[typing.Sequence[str]] = None,
    ) -> bool: ...
    def scale(
        self,
        dest: Pixbuf,
        dest_x: int,
        dest_y: int,
        dest_width: int,
        dest_height: int,
        offset_x: float,
        offset_y: float,
        scale_x: float,
        scale_y: float,
        interp_type: InterpType,
    ) -> None: ...
    def scale_simple(
        self, dest_width: int, dest_height: int, interp_type: InterpType
    ) -> typing.Optional[Pixbuf]: ...
    def set_option(self, key: str, value: str) -> bool: ...

class PixbufAnimation(GObject.Object):
    """
    :Constructors:

    ::

        PixbufAnimation(**properties)
        new_from_file(filename:str) -> GdkPixbuf.PixbufAnimation or None
        new_from_resource(resource_path:str) -> GdkPixbuf.PixbufAnimation or None
        new_from_stream(stream:Gio.InputStream, cancellable:Gio.Cancellable=None) -> GdkPixbuf.PixbufAnimation or None
        new_from_stream_finish(async_result:Gio.AsyncResult) -> GdkPixbuf.PixbufAnimation or None

    Object GdkPixbufAnimation

    Signals from GObject:
      notify (GParam)
    """

    parent_instance: GObject.Object = ...
    def do_get_iter(
        self, start_time: typing.Optional[GLib.TimeVal] = None
    ) -> PixbufAnimationIter: ...
    def do_get_size(self, width: int, height: int) -> None: ...
    def do_get_static_image(self) -> Pixbuf: ...
    def do_is_static_image(self) -> bool: ...
    def get_height(self) -> int: ...
    def get_iter(
        self, start_time: typing.Optional[GLib.TimeVal] = None
    ) -> PixbufAnimationIter: ...
    def get_static_image(self) -> Pixbuf: ...
    def get_width(self) -> int: ...
    def is_static_image(self) -> bool: ...
    @classmethod
    def new_from_file(cls, filename: str) -> typing.Optional[PixbufAnimation]: ...
    @classmethod
    def new_from_resource(
        cls, resource_path: str
    ) -> typing.Optional[PixbufAnimation]: ...
    @classmethod
    def new_from_stream(
        cls,
        stream: Gio.InputStream,
        cancellable: typing.Optional[Gio.Cancellable] = None,
    ) -> typing.Optional[PixbufAnimation]: ...
    @staticmethod
    def new_from_stream_async(
        stream: Gio.InputStream,
        cancellable: typing.Optional[Gio.Cancellable] = None,
        callback: typing.Optional[typing.Callable[..., None]] = None,
        *user_data: typing.Any,
    ) -> None: ...
    @classmethod
    def new_from_stream_finish(
        cls, async_result: Gio.AsyncResult
    ) -> PixbufAnimation: ...

class PixbufAnimationClass(GObject.GPointer):
    """
    :Constructors:

    ::

        PixbufAnimationClass()
    """

    parent_class: GObject.ObjectClass = ...
    is_static_image: typing.Callable[[PixbufAnimation], bool] = ...
    get_static_image: typing.Callable[[PixbufAnimation], Pixbuf] = ...
    get_size: typing.Callable[[PixbufAnimation, int, int], None] = ...
    get_iter: typing.Callable[
        [PixbufAnimation, typing.Optional[GLib.TimeVal]], PixbufAnimationIter
    ] = ...

class PixbufAnimationIter(GObject.Object):
    """
    :Constructors:

    ::

        PixbufAnimationIter(**properties)

    Object GdkPixbufAnimationIter

    Signals from GObject:
      notify (GParam)
    """

    parent_instance: GObject.Object = ...
    def advance(self, current_time: typing.Optional[GLib.TimeVal] = None) -> bool: ...
    def do_advance(
        self, current_time: typing.Optional[GLib.TimeVal] = None
    ) -> bool: ...
    def do_get_delay_time(self) -> int: ...
    def do_get_pixbuf(self) -> Pixbuf: ...
    def do_on_currently_loading_frame(self) -> bool: ...
    def get_delay_time(self) -> int: ...
    def get_pixbuf(self) -> Pixbuf: ...
    def on_currently_loading_frame(self) -> bool: ...

class PixbufAnimationIterClass(GObject.GPointer):
    """
    :Constructors:

    ::

        PixbufAnimationIterClass()
    """

    parent_class: GObject.ObjectClass = ...
    get_delay_time: typing.Callable[[PixbufAnimationIter], int] = ...
    get_pixbuf: typing.Callable[[PixbufAnimationIter], Pixbuf] = ...
    on_currently_loading_frame: typing.Callable[[PixbufAnimationIter], bool] = ...
    advance: typing.Callable[
        [PixbufAnimationIter, typing.Optional[GLib.TimeVal]], bool
    ] = ...

class PixbufFormat(GObject.GBoxed):
    """
    :Constructors:

    ::

        PixbufFormat()
    """

    name: str = ...
    signature: PixbufModulePattern = ...
    domain: str = ...
    description: str = ...
    mime_types: list[str] = ...
    extensions: list[str] = ...
    flags: int = ...
    disabled: bool = ...
    license: str = ...
    def copy(self) -> typing.Optional[PixbufFormat]: ...
    def free(self) -> None: ...
    def get_description(self) -> typing.Optional[str]: ...
    def get_extensions(self) -> typing.Optional[list[str]]: ...
    def get_license(self) -> typing.Optional[str]: ...
    def get_mime_types(self) -> typing.Optional[list[str]]: ...
    def get_name(self) -> typing.Optional[str]: ...
    def is_disabled(self) -> bool: ...
    def is_save_option_supported(self, option_key: str) -> bool: ...
    def is_scalable(self) -> bool: ...
    def is_writable(self) -> bool: ...
    def set_disabled(self, disabled: bool) -> None: ...

class PixbufLoader(GObject.Object):
    """
    :Constructors:

    ::

        PixbufLoader(**properties)
        new() -> GdkPixbuf.PixbufLoader
        new_with_mime_type(mime_type:str) -> GdkPixbuf.PixbufLoader
        new_with_type(image_type:str) -> GdkPixbuf.PixbufLoader

    Object GdkPixbufLoader

    Signals from GdkPixbufLoader:
      size-prepared (gint, gint)
      area-prepared ()
      area-updated (gint, gint, gint, gint)
      closed ()

    Signals from GObject:
      notify (GParam)
    """

    parent_instance: GObject.Object = ...
    priv: None = ...
    def close(self) -> bool: ...
    def do_area_prepared(self) -> None: ...
    def do_area_updated(self, x: int, y: int, width: int, height: int) -> None: ...
    def do_closed(self) -> None: ...
    def do_size_prepared(self, width: int, height: int) -> None: ...
    def get_animation(self) -> typing.Optional[PixbufAnimation]: ...
    def get_format(self) -> typing.Optional[PixbufFormat]: ...
    def get_pixbuf(self) -> typing.Optional[Pixbuf]: ...
    @classmethod
    def new(cls) -> PixbufLoader: ...
    @classmethod
    def new_with_mime_type(cls, mime_type: str) -> PixbufLoader: ...
    @classmethod
    def new_with_type(cls, image_type: str) -> PixbufLoader: ...
    def set_size(self, width: int, height: int) -> None: ...
    def write(self, buf: typing.Sequence[int]) -> bool: ...
    def write_bytes(self, buffer: GLib.Bytes) -> bool: ...

class PixbufLoaderClass(GObject.GPointer):
    """
    :Constructors:

    ::

        PixbufLoaderClass()
    """

    parent_class: GObject.ObjectClass = ...
    size_prepared: typing.Callable[[PixbufLoader, int, int], None] = ...
    area_prepared: typing.Callable[[PixbufLoader], None] = ...
    area_updated: typing.Callable[[PixbufLoader, int, int, int, int], None] = ...
    closed: typing.Callable[[PixbufLoader], None] = ...

class PixbufModule(GObject.GPointer):
    """
    :Constructors:

    ::

        PixbufModule()
    """

    module_name: str = ...
    module_path: str = ...
    module: GModule.Module = ...
    info: PixbufFormat = ...
    load: typing.Callable[[None], Pixbuf] = ...
    load_xpm_data: typing.Callable[[typing.Sequence[str]], Pixbuf] = ...
    begin_load: None = ...
    stop_load: typing.Callable[[None], bool] = ...
    load_increment: typing.Callable[[None, typing.Sequence[int]], bool] = ...
    load_animation: typing.Callable[[None], PixbufAnimation] = ...
    save: typing.Callable[
        [
            None,
            Pixbuf,
            typing.Optional[typing.Sequence[str]],
            typing.Optional[typing.Sequence[str]],
        ],
        bool,
    ] = ...
    save_to_callback: None = ...
    is_save_option_supported: typing.Callable[[str], bool] = ...
    _reserved1: None = ...
    _reserved2: None = ...
    _reserved3: None = ...
    _reserved4: None = ...

class PixbufModulePattern(GObject.GPointer):
    """
    :Constructors:

    ::

        PixbufModulePattern()
    """

    prefix: str = ...
    mask: str = ...
    relevance: int = ...

class PixbufNonAnim(PixbufAnimation):
    """
    :Constructors:

    ::

        PixbufNonAnim(**properties)
        new(pixbuf:GdkPixbuf.Pixbuf) -> GdkPixbuf.PixbufAnimation

    Object GdkPixbufNonAnim

    Signals from GObject:
      notify (GParam)
    """
    @classmethod
    def new(cls, pixbuf: Pixbuf) -> PixbufNonAnim: ...

class PixbufSimpleAnim(PixbufAnimation):
    """
    :Constructors:

    ::

        PixbufSimpleAnim(**properties)
        new(width:int, height:int, rate:float) -> GdkPixbuf.PixbufSimpleAnim

    Object GdkPixbufSimpleAnim

    Properties from GdkPixbufSimpleAnim:
      loop -> gboolean: Loop
        Whether the animation should loop when it reaches the end

    Signals from GObject:
      notify (GParam)
    """
    class Props(PixbufAnimation.Props):
        loop: bool

    props: Props = ...
    def __init__(self, loop: bool = ...) -> None: ...
    def add_frame(self, pixbuf: Pixbuf) -> None: ...
    def get_loop(self) -> bool: ...
    @classmethod
    def new(cls, width: int, height: int, rate: float) -> PixbufSimpleAnim: ...
    def set_loop(self, loop: bool) -> None: ...

class PixbufSimpleAnimClass(GObject.GPointer): ...
class PixbufSimpleAnimIter(PixbufAnimationIter): ...

class PixbufFormatFlags(GObject.GFlags):
    SCALABLE = 2
    THREADSAFE = 4
    WRITABLE = 1

class Colorspace(GObject.GEnum):
    RGB = 0

class InterpType(GObject.GEnum):
    BILINEAR = 2
    HYPER = 3
    NEAREST = 0
    TILES = 1

class PixbufAlphaMode(GObject.GEnum):
    BILEVEL = 0
    FULL = 1

class PixbufError(GObject.GEnum):
    BAD_OPTION = 2
    CORRUPT_IMAGE = 0
    FAILED = 5
    INCOMPLETE_ANIMATION = 6
    INSUFFICIENT_MEMORY = 1
    UNKNOWN_TYPE = 3
    UNSUPPORTED_OPERATION = 4
    @staticmethod
    def quark() -> int: ...

class PixbufRotation(GObject.GEnum):
    CLOCKWISE = 270
    COUNTERCLOCKWISE = 90
    NONE = 0
    UPSIDEDOWN = 180



from typing import Any
from typing import Callable
from typing import Literal
from typing import Optional
from typing import Sequence
from typing import Tuple
from typing import Type
from typing import TypeVar

from gi.repository import Gio
from gi.repository import GLib
from gi.repository import GObject

COMMIT_GVARIANT_STRING: str = "(a{sv}aya(say)sstayay)"
COMMIT_META_KEY_ARCHITECTURE: str = "ostree.architecture"
COMMIT_META_KEY_COLLECTION_BINDING: str = "ostree.collection-binding"
COMMIT_META_KEY_ENDOFLIFE: str = "ostree.endoflife"
COMMIT_META_KEY_ENDOFLIFE_REBASE: str = "ostree.endoflife-rebase"
COMMIT_META_KEY_REF_BINDING: str = "ostree.ref-binding"
COMMIT_META_KEY_SOURCE_TITLE: str = "ostree.source-title"
COMMIT_META_KEY_VERSION: str = "version"
DIRMETA_GVARIANT_STRING: str = "(uuua(ayay))"
FILEMETA_GVARIANT_STRING: str = "(uuua(ayay))"
GPG_KEY_GVARIANT_STRING: str = "(aa{sv}aa{sv}a{sv})"
MAX_METADATA_SIZE: int = 134217728
MAX_METADATA_WARN_SIZE: int = 7340032
METADATA_KEY_BOOTABLE: str = "ostree.bootable"
METADATA_KEY_LINUX: str = "ostree.linux"
META_KEY_DEPLOY_COLLECTION_ID: str = "ostree.deploy-collection-id"
ORIGIN_TRANSIENT_GROUP: str = "libostree-transient"
PATH_BOOTED: str = "/run/ostree-booted"
RELEASE_VERSION: int = 4
REPO_METADATA_REF: str = "ostree-metadata"
SHA256_DIGEST_LEN: int = 32
SHA256_STRING_LEN: int = 64
SIGN_NAME_ED25519: str = "ed25519"
SUMMARY_GVARIANT_STRING: str = "(a(s(taya{sv}))a{sv})"
SUMMARY_SIG_GVARIANT_STRING: str = "a{sv}"
TIMESTAMP: int = 0
TREE_GVARIANT_STRING: str = "(a(say)a(sayay))"
VERSION: float = 2023.4
VERSION_S: str = "2023.4"
YEAR_VERSION: int = 2023
_lock = ...  # FIXME Constant
_namespace: str = "OSTree"
_version: str = "1.0"

def break_hardlink(
    dfd: int,
    path: str,
    skip_xattrs: bool,
    cancellable: Optional[Gio.Cancellable] = None,
) -> bool: ...
def check_version(required_year: int, required_release: int) -> bool: ...
def checksum_b64_from_bytes(csum: Sequence[int]) -> str: ...
def checksum_b64_to_bytes(checksum: str) -> bytes: ...
def checksum_bytes_peek(bytes: GLib.Variant) -> bytes: ...
def checksum_bytes_peek_validate(bytes: GLib.Variant) -> bytes: ...
def checksum_file(
    f: Gio.File, objtype: ObjectType, cancellable: Optional[Gio.Cancellable] = None
) -> Tuple[bool, bytes]: ...
def checksum_file_async(
    f: Gio.File,
    objtype: ObjectType,
    io_priority: int,
    cancellable: Optional[Gio.Cancellable] = None,
    callback: Optional[Callable[..., None]] = None,
    *user_data: Any,
) -> None: ...
def checksum_file_async_finish(
    f: Gio.File, result: Gio.AsyncResult
) -> Tuple[bool, bytes]: ...
def checksum_file_at(
    dfd: int,
    path: str,
    stbuf: None,
    objtype: ObjectType,
    flags: ChecksumFlags,
    out_checksum: str,
    cancellable: Optional[Gio.Cancellable] = None,
) -> bool: ...
def checksum_file_from_input(
    file_info: Gio.FileInfo,
    xattrs: Optional[GLib.Variant],
    in_: Optional[Gio.InputStream],
    objtype: ObjectType,
    cancellable: Optional[Gio.Cancellable] = None,
) -> Tuple[bool, bytes]: ...
def checksum_from_bytes(csum: Sequence[int]) -> str: ...
def checksum_from_bytes_v(csum_v: GLib.Variant) -> str: ...
def checksum_inplace_to_bytes(checksum: str, buf: int) -> None: ...
def checksum_to_bytes(checksum: str) -> bytes: ...
def checksum_to_bytes_v(checksum: str) -> GLib.Variant: ...
def cmp_checksum_bytes(a: int, b: int) -> int: ...
def collection_ref_dupv(refs: Sequence[CollectionRef]) -> list[CollectionRef]: ...
def collection_ref_freev(refs: Sequence[CollectionRef]) -> None: ...
def commit_get_content_checksum(commit_variant: GLib.Variant) -> Optional[str]: ...
def commit_get_object_sizes(
    commit_variant: GLib.Variant,
) -> Tuple[bool, list[CommitSizesEntry]]: ...
def commit_get_parent(commit_variant: GLib.Variant) -> Optional[str]: ...
def commit_get_timestamp(commit_variant: GLib.Variant) -> int: ...
def commit_metadata_for_bootable(
    root: Gio.File,
    dict: GLib.VariantDict,
    cancellable: Optional[Gio.Cancellable] = None,
) -> bool: ...
def content_file_parse(
    compressed: bool,
    content_path: Gio.File,
    trusted: bool,
    cancellable: Optional[Gio.Cancellable] = None,
) -> Tuple[bool, Gio.InputStream, Gio.FileInfo, GLib.Variant]: ...
def content_file_parse_at(
    compressed: bool,
    parent_dfd: int,
    path: str,
    trusted: bool,
    cancellable: Optional[Gio.Cancellable] = None,
) -> Tuple[bool, Gio.InputStream, Gio.FileInfo, GLib.Variant]: ...
def content_stream_parse(
    compressed: bool,
    input: Gio.InputStream,
    input_length: int,
    trusted: bool,
    cancellable: Optional[Gio.Cancellable] = None,
) -> Tuple[bool, Gio.InputStream, Gio.FileInfo, GLib.Variant]: ...
def create_directory_metadata(
    dir_info: Gio.FileInfo, xattrs: Optional[GLib.Variant] = None
) -> GLib.Variant: ...
def diff_dirs(
    flags: DiffFlags,
    a: Gio.File,
    b: Gio.File,
    modified: Sequence[DiffItem],
    removed: Sequence[Gio.File],
    added: Sequence[Gio.File],
    cancellable: Optional[Gio.Cancellable] = None,
) -> bool: ...
def diff_dirs_with_options(
    flags: DiffFlags,
    a: Gio.File,
    b: Gio.File,
    modified: Sequence[DiffItem],
    removed: Sequence[Gio.File],
    added: Sequence[Gio.File],
    options: Optional[DiffDirsOptions] = None,
    cancellable: Optional[Gio.Cancellable] = None,
) -> bool: ...
def diff_print(
    a: Gio.File,
    b: Gio.File,
    modified: Sequence[DiffItem],
    removed: Sequence[Gio.File],
    added: Sequence[Gio.File],
) -> None: ...
def fs_get_all_xattrs(
    fd: int, cancellable: Optional[Gio.Cancellable] = None
) -> GLib.Variant: ...
def fs_get_all_xattrs_at(
    dfd: int, path: str, cancellable: Optional[Gio.Cancellable] = None
) -> GLib.Variant: ...
def gpg_error_quark() -> int: ...
def hash_object_name(a: None) -> int: ...
def kernel_args_cleanup(loc: None) -> None: ...
def metadata_variant_type(objtype: ObjectType) -> GLib.VariantType: ...
def object_from_string(str: str) -> Tuple[str, ObjectType]: ...
def object_name_deserialize(variant: GLib.Variant) -> Tuple[str, ObjectType]: ...
def object_name_serialize(checksum: str, objtype: ObjectType) -> GLib.Variant: ...
def object_to_string(checksum: str, objtype: ObjectType) -> str: ...
def object_type_from_string(str: str) -> ObjectType: ...
def object_type_to_string(objtype: ObjectType) -> str: ...
def parse_refspec(refspec: str) -> Tuple[bool, str, str]: ...
def raw_file_to_archive_z2_stream(
    input: Gio.InputStream,
    file_info: Gio.FileInfo,
    xattrs: Optional[GLib.Variant] = None,
    cancellable: Optional[Gio.Cancellable] = None,
) -> Tuple[bool, Gio.InputStream]: ...
def raw_file_to_archive_z2_stream_with_options(
    input: Gio.InputStream,
    file_info: Gio.FileInfo,
    xattrs: Optional[GLib.Variant] = None,
    options: Optional[GLib.Variant] = None,
    cancellable: Optional[Gio.Cancellable] = None,
) -> Tuple[bool, Gio.InputStream]: ...
def raw_file_to_content_stream(
    input: Gio.InputStream,
    file_info: Gio.FileInfo,
    xattrs: Optional[GLib.Variant] = None,
    cancellable: Optional[Gio.Cancellable] = None,
) -> Tuple[bool, Gio.InputStream, int]: ...
def repo_commit_traverse_iter_cleanup(p: None) -> None: ...
def repo_finder_resolve_all_async(
    finders: Sequence[RepoFinder],
    refs: Sequence[CollectionRef],
    parent_repo: Repo,
    cancellable: Optional[Gio.Cancellable] = None,
    callback: Optional[Callable[..., None]] = None,
    *user_data: Any,
) -> None: ...
def repo_finder_resolve_all_finish(
    result: Gio.AsyncResult,
) -> list[RepoFinderResult]: ...
def repo_finder_result_freev(results: Sequence[RepoFinderResult]) -> None: ...
def sign_get_all() -> list[Sign]: ...
def sign_get_by_name(name: str) -> Sign: ...
def validate_checksum_string(sha256: str) -> bool: ...
def validate_collection_id(collection_id: Optional[str] = None) -> bool: ...
def validate_remote_name(remote_name: str) -> bool: ...
def validate_rev(rev: str) -> bool: ...
def validate_structureof_checksum_string(checksum: str) -> bool: ...
def validate_structureof_commit(commit: GLib.Variant) -> bool: ...
def validate_structureof_csum_v(checksum: GLib.Variant) -> bool: ...
def validate_structureof_dirmeta(dirmeta: GLib.Variant) -> bool: ...
def validate_structureof_dirtree(dirtree: GLib.Variant) -> bool: ...
def validate_structureof_file_mode(mode: int) -> bool: ...
def validate_structureof_objtype(objtype: int) -> bool: ...

class AsyncProgress(GObject.Object):
    """
    :Constructors:

    ::

        AsyncProgress(**properties)
        new() -> OSTree.AsyncProgress

    Object OstreeAsyncProgress

    Signals from OstreeAsyncProgress:
      changed ()

    Signals from GObject:
      notify (GParam)
    """

    def copy_state(self, dest: AsyncProgress) -> None: ...
    def do_changed(self, *user_data: Any) -> None: ...
    def finish(self) -> None: ...
    def get_status(self) -> Optional[str]: ...
    def get_uint(self, key: str) -> int: ...
    def get_uint64(self, key: str) -> int: ...
    def get_variant(self, key: str) -> Optional[GLib.Variant]: ...
    @classmethod
    def new(cls) -> AsyncProgress: ...
    def set_status(self, status: Optional[str] = None) -> None: ...
    def set_uint(self, key: str, value: int) -> None: ...
    def set_uint64(self, key: str, value: int) -> None: ...
    def set_variant(self, key: str, value: GLib.Variant) -> None: ...

class AsyncProgressClass(GObject.GPointer):
    """
    :Constructors:

    ::

        AsyncProgressClass()
    """

    parent_class: GObject.ObjectClass = ...
    changed: Callable[..., None] = ...

class BootconfigParser(GObject.Object):
    """
    :Constructors:

    ::

        BootconfigParser(**properties)
        new() -> OSTree.BootconfigParser

    Object OstreeBootconfigParser

    Signals from GObject:
      notify (GParam)
    """

    def clone(self) -> BootconfigParser: ...
    def get(self, key: str) -> Optional[str]: ...
    def get_overlay_initrds(self) -> Optional[list[str]]: ...
    @classmethod
    def new(cls) -> BootconfigParser: ...
    def parse(
        self, path: Gio.File, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...
    def parse_at(
        self, dfd: int, path: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...
    def set(self, key: str, value: str) -> None: ...
    def set_overlay_initrds(self, initrds: Optional[Sequence[str]] = None) -> None: ...
    def write(
        self, output: Gio.File, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...
    def write_at(
        self, dfd: int, path: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...

class CollectionRef(GObject.GBoxed):
    """
    :Constructors:

    ::

        CollectionRef()
        new(collection_id:str=None, ref_name:str) -> OSTree.CollectionRef
    """

    collection_id: str = ...
    ref_name: str = ...
    def dup(self) -> CollectionRef: ...
    @staticmethod
    def dupv(refs: Sequence[CollectionRef]) -> list[CollectionRef]: ...
    def equal(self, ref2: CollectionRef) -> bool: ...
    def free(self) -> None: ...
    @staticmethod
    def freev(refs: Sequence[CollectionRef]) -> None: ...
    def hash(self) -> int: ...
    @classmethod
    def new(cls, collection_id: Optional[str], ref_name: str) -> CollectionRef: ...

class CommitSizesEntry(GObject.GBoxed):
    """
    :Constructors:

    ::

        CommitSizesEntry()
        new(checksum:str, objtype:OSTree.ObjectType, unpacked:int, archived:int) -> OSTree.CommitSizesEntry or None
    """

    checksum: str = ...
    objtype: ObjectType = ...
    unpacked: int = ...
    archived: int = ...
    def copy(self) -> Optional[CommitSizesEntry]: ...
    def free(self) -> None: ...
    @classmethod
    def new(
        cls, checksum: str, objtype: ObjectType, unpacked: int, archived: int
    ) -> Optional[CommitSizesEntry]: ...

class ContentWriter(Gio.OutputStream):
    """
    :Constructors:

    ::

        ContentWriter(**properties)

    Object OstreeContentWriter

    Signals from GObject:
      notify (GParam)
    """

    def finish(self, cancellable: Optional[Gio.Cancellable] = None) -> str: ...

class ContentWriterClass(GObject.GPointer):
    """
    :Constructors:

    ::

        ContentWriterClass()
    """

    parent_class: Gio.OutputStreamClass = ...

class Deployment(GObject.Object):
    """
    :Constructors:

    ::

        Deployment(**properties)
        new(index:int, osname:str, csum:str, deployserial:int, bootcsum:str=None, bootserial:int) -> OSTree.Deployment

    Object OstreeDeployment

    Signals from GObject:
      notify (GParam)
    """

    def clone(self) -> Deployment: ...
    def equal(self, bp: Deployment) -> bool: ...
    def get_bootconfig(self) -> Optional[BootconfigParser]: ...
    def get_bootcsum(self) -> str: ...
    def get_bootserial(self) -> int: ...
    def get_csum(self) -> str: ...
    def get_deployserial(self) -> int: ...
    def get_index(self) -> int: ...
    def get_origin(self) -> Optional[GLib.KeyFile]: ...
    def get_origin_relpath(self) -> str: ...
    def get_osname(self) -> str: ...
    def get_unlocked(self) -> DeploymentUnlockedState: ...
    def hash(self) -> int: ...
    def is_pinned(self) -> bool: ...
    def is_staged(self) -> bool: ...
    @classmethod
    def new(
        cls,
        index: int,
        osname: str,
        csum: str,
        deployserial: int,
        bootcsum: Optional[str],
        bootserial: int,
    ) -> Deployment: ...
    @staticmethod
    def origin_remove_transient_state(origin: GLib.KeyFile) -> None: ...
    def set_bootconfig(self, bootconfig: Optional[BootconfigParser] = None) -> None: ...
    def set_bootserial(self, index: int) -> None: ...
    def set_index(self, index: int) -> None: ...
    def set_origin(self, origin: Optional[GLib.KeyFile] = None) -> None: ...
    @staticmethod
    def unlocked_state_to_string(state: DeploymentUnlockedState) -> str: ...

class DiffDirsOptions(GObject.GPointer):
    """
    :Constructors:

    ::

        DiffDirsOptions()
    """

    owner_uid: int = ...
    owner_gid: int = ...
    devino_to_csum_cache: RepoDevInoCache = ...
    unused_bools: list[bool] = ...
    unused_ints: list[int] = ...
    unused_ptrs: list[None] = ...

class DiffItem(GObject.GBoxed):
    """
    :Constructors:

    ::

        DiffItem()
    """

    refcount: int = ...
    src: Gio.File = ...
    target: Gio.File = ...
    src_info: Gio.FileInfo = ...
    target_info: Gio.FileInfo = ...
    src_checksum: str = ...
    target_checksum: str = ...
    def ref(self) -> DiffItem: ...
    def unref(self) -> None: ...

class GpgVerifyResult(GObject.Object, Gio.Initable):
    """
    :Constructors:

    ::

        GpgVerifyResult(**properties)

    Object OstreeGpgVerifyResult

    Signals from GObject:
      notify (GParam)
    """

    def count_all(self) -> int: ...
    def count_valid(self) -> int: ...
    def describe(
        self,
        signature_index: int,
        output_buffer: GLib.String,
        line_prefix: Optional[str],
        flags: GpgSignatureFormatFlags,
    ) -> None: ...
    @staticmethod
    def describe_variant(
        variant: GLib.Variant,
        output_buffer: GLib.String,
        line_prefix: Optional[str],
        flags: GpgSignatureFormatFlags,
    ) -> None: ...
    def get(
        self, signature_index: int, attrs: Sequence[GpgSignatureAttr]
    ) -> GLib.Variant: ...
    def get_all(self, signature_index: int) -> GLib.Variant: ...
    def lookup(self, key_id: str) -> Tuple[bool, int]: ...
    def require_valid_signature(self) -> bool: ...

class KernelArgs(GObject.GPointer):
    def append(self, arg: str) -> None: ...
    def append_argv(self, argv: Sequence[str]) -> None: ...
    def append_argv_filtered(
        self, argv: Sequence[str], prefixes: Sequence[str]
    ) -> None: ...
    def append_if_missing(self, arg: str) -> None: ...
    def append_proc_cmdline(
        self, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...
    @staticmethod
    def cleanup(loc: None) -> None: ...
    def contains(self, arg: str) -> bool: ...
    def delete(self, arg: str) -> bool: ...
    def delete_if_present(self, arg: str) -> bool: ...
    def delete_key_entry(self, key: str) -> bool: ...
    def free(self) -> None: ...
    def get_last_value(self, key: str) -> Optional[str]: ...
    def new_replace(self, arg: str) -> bool: ...
    def parse_append(self, options: str) -> None: ...
    def replace(self, arg: str) -> None: ...
    def replace_argv(self, argv: str) -> None: ...
    def replace_take(self, arg: str) -> None: ...
    def to_string(self) -> str: ...
    def to_strv(self) -> list[str]: ...

class KernelArgsEntry(GObject.GPointer): ...

class MutableTree(GObject.Object):
    """
    :Constructors:

    ::

        MutableTree(**properties)
        new() -> OSTree.MutableTree
        new_from_checksum(repo:OSTree.Repo, contents_checksum:str, metadata_checksum:str) -> OSTree.MutableTree
        new_from_commit(repo:OSTree.Repo, rev:str) -> OSTree.MutableTree

    Object OstreeMutableTree

    Signals from GObject:
      notify (GParam)
    """

    def check_error(self) -> bool: ...
    def ensure_dir(self, name: str) -> Tuple[bool, MutableTree]: ...
    def ensure_parent_dirs(
        self, split_path: Sequence[str], metadata_checksum: str
    ) -> Tuple[bool, MutableTree]: ...
    def fill_empty_from_dirtree(
        self, repo: Repo, contents_checksum: str, metadata_checksum: str
    ) -> bool: ...
    def get_contents_checksum(self) -> str: ...
    def get_files(self) -> dict[str, str]: ...
    def get_metadata_checksum(self) -> str: ...
    def get_subdirs(self) -> dict[str, MutableTree]: ...
    def lookup(self, name: str) -> Tuple[bool, str, MutableTree]: ...
    @classmethod
    def new(cls) -> MutableTree: ...
    @classmethod
    def new_from_checksum(
        cls, repo: Repo, contents_checksum: str, metadata_checksum: str
    ) -> MutableTree: ...
    @classmethod
    def new_from_commit(cls, repo: Repo, rev: str) -> MutableTree: ...
    def remove(self, name: str, allow_noent: bool) -> bool: ...
    def replace_file(self, name: str, checksum: str) -> bool: ...
    def set_contents_checksum(self, checksum: str) -> None: ...
    def set_metadata_checksum(self, checksum: str) -> None: ...
    def walk(
        self, split_path: Sequence[str], start: int
    ) -> Tuple[bool, MutableTree]: ...

class MutableTreeClass(GObject.GPointer):
    """
    :Constructors:

    ::

        MutableTreeClass()
    """

    parent_class: GObject.ObjectClass = ...

class MutableTreeIter(GObject.GPointer):
    """
    :Constructors:

    ::

        MutableTreeIter()
    """

    in_files: bool = ...
    iter: dict[None, None] = ...

class Remote(GObject.GBoxed):
    def get_name(self) -> str: ...
    def get_url(self) -> Optional[str]: ...
    def ref(self) -> Remote: ...
    def unref(self) -> None: ...

class Repo(GObject.Object):
    """
    :Constructors:

    ::

        Repo(**properties)
        new(path:Gio.File) -> OSTree.Repo
        new_default() -> OSTree.Repo
        new_for_sysroot_path(repo_path:Gio.File, sysroot_path:Gio.File) -> OSTree.Repo

    Object OstreeRepo

    Signals from OstreeRepo:
      gpg-verify-result (gchararray, OstreeGpgVerifyResult)

    Properties from OstreeRepo:
      path -> GFile: Path
        Path
      remotes-config-dir -> gchararray:

      sysroot-path -> GFile:


    Signals from GObject:
      notify (GParam)
    """

    class Props:
        path: Gio.File
        remotes_config_dir: str
        sysroot_path: Gio.File

    props: Props = ...
    def __init__(
        self,
        path: Gio.File = ...,
        remotes_config_dir: str = ...,
        sysroot_path: Gio.File = ...,
    ): ...
    def abort_transaction(
        self, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...
    def add_gpg_signature_summary(
        self,
        key_id: Sequence[str],
        homedir: Optional[str] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def append_gpg_signature(
        self,
        commit_checksum: str,
        signature_bytes: GLib.Bytes,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def checkout_at(
        self,
        options: Optional[RepoCheckoutAtOptions],
        destination_dfd: int,
        destination_path: str,
        commit: str,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def checkout_gc(self, cancellable: Optional[Gio.Cancellable] = None) -> bool: ...
    def checkout_tree(
        self,
        mode: RepoCheckoutMode,
        overwrite_mode: RepoCheckoutOverwriteMode,
        destination: Gio.File,
        source: RepoFile,
        source_info: Gio.FileInfo,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def commit_add_composefs_metadata(
        self,
        format_version: int,
        dict: GLib.VariantDict,
        repo_root: RepoFile,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def commit_transaction(
        self, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, RepoTransactionStats]: ...
    def copy_config(self) -> GLib.KeyFile: ...
    def create(
        self, mode: RepoMode, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...
    @staticmethod
    def create_at(
        dfd: int,
        path: str,
        mode: RepoMode,
        options: Optional[GLib.Variant] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Repo: ...
    def delete_object(
        self,
        objtype: ObjectType,
        sha256: str,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def equal(self, b: Repo) -> bool: ...
    def find_remotes_async(
        self,
        refs: Sequence[CollectionRef],
        options: Optional[GLib.Variant],
        finders: Sequence[RepoFinder],
        progress: Optional[AsyncProgress] = None,
        cancellable: Optional[Gio.Cancellable] = None,
        callback: Optional[Callable[..., None]] = None,
        *user_data: Any,
    ) -> None: ...
    def find_remotes_finish(
        self, result: Gio.AsyncResult
    ) -> list[RepoFinderResult]: ...
    def fsck_object(
        self,
        objtype: ObjectType,
        sha256: str,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def get_bootloader(self) -> str: ...
    def get_collection_id(self) -> Optional[str]: ...
    def get_config(self) -> GLib.KeyFile: ...
    def get_default_repo_finders(self) -> list[str]: ...
    def get_dfd(self) -> int: ...
    def get_disable_fsync(self) -> bool: ...
    def get_min_free_space_bytes(self) -> Tuple[bool, int]: ...
    def get_mode(self) -> RepoMode: ...
    def get_parent(self) -> Optional[Repo]: ...
    def get_path(self) -> Gio.File: ...
    def get_remote_boolean_option(
        self, remote_name: str, option_name: str, default_value: bool
    ) -> Tuple[bool, bool]: ...
    def get_remote_list_option(
        self, remote_name: str, option_name: str
    ) -> Tuple[bool, list[str]]: ...
    def get_remote_option(
        self, remote_name: str, option_name: str, default_value: Optional[str] = None
    ) -> Tuple[bool, str]: ...
    def gpg_sign_data(
        self,
        data: GLib.Bytes,
        old_signatures: Optional[GLib.Bytes],
        key_id: Sequence[str],
        homedir: Optional[str] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, GLib.Bytes]: ...
    def gpg_verify_data(
        self,
        remote_name: Optional[str],
        data: GLib.Bytes,
        signatures: GLib.Bytes,
        keyringdir: Optional[Gio.File] = None,
        extra_keyring: Optional[Gio.File] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> GpgVerifyResult: ...
    def has_object(
        self,
        objtype: ObjectType,
        checksum: str,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, bool]: ...
    def hash(self) -> int: ...
    def import_object_from(
        self,
        source: Repo,
        objtype: ObjectType,
        checksum: str,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def import_object_from_with_trust(
        self,
        source: Repo,
        objtype: ObjectType,
        checksum: str,
        trusted: bool,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def is_system(self) -> bool: ...
    def is_writable(self) -> bool: ...
    def list_collection_refs(
        self,
        match_collection_id: Optional[str],
        flags: RepoListRefsExtFlags,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, dict[CollectionRef, str]]: ...
    def list_commit_objects_starting_with(
        self, start: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, dict[GLib.Variant, GLib.Variant]]: ...
    def list_objects(
        self, flags: RepoListObjectsFlags, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, dict[GLib.Variant, GLib.Variant]]: ...
    def list_refs(
        self,
        refspec_prefix: Optional[str] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, dict[str, str]]: ...
    def list_refs_ext(
        self,
        refspec_prefix: Optional[str],
        flags: RepoListRefsExtFlags,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, dict[str, str]]: ...
    def list_static_delta_indexes(
        self, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, list[str]]: ...
    def list_static_delta_names(
        self, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, list[str]]: ...
    def load_commit(
        self, checksum: str
    ) -> Tuple[bool, GLib.Variant, RepoCommitState]: ...
    def load_file(
        self, checksum: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, Gio.InputStream, Gio.FileInfo, GLib.Variant]: ...
    def load_object_stream(
        self,
        objtype: ObjectType,
        checksum: str,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, Gio.InputStream, int]: ...
    def load_variant(
        self, objtype: ObjectType, sha256: str
    ) -> Tuple[bool, GLib.Variant]: ...
    def load_variant_if_exists(
        self, objtype: ObjectType, sha256: str
    ) -> Tuple[bool, GLib.Variant]: ...
    def lock_pop(
        self, lock_type: RepoLockType, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...
    def lock_push(
        self, lock_type: RepoLockType, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...
    def mark_commit_partial(self, checksum: str, is_partial: bool) -> bool: ...
    def mark_commit_partial_reason(
        self, checksum: str, is_partial: bool, in_state: RepoCommitState
    ) -> bool: ...
    @staticmethod
    def mode_from_string(mode: str) -> Tuple[bool, RepoMode]: ...
    @classmethod
    def new(cls, path: Gio.File) -> Repo: ...
    @classmethod
    def new_default(cls) -> Repo: ...
    @classmethod
    def new_for_sysroot_path(
        cls, repo_path: Gio.File, sysroot_path: Gio.File
    ) -> Repo: ...
    def open(self, cancellable: Optional[Gio.Cancellable] = None) -> bool: ...
    @staticmethod
    def open_at(
        dfd: int, path: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> Repo: ...
    def prepare_transaction(
        self, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, bool]: ...
    def prune(
        self,
        flags: RepoPruneFlags,
        depth: int,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, int, int, int]: ...
    def prune_from_reachable(
        self, options: RepoPruneOptions, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, int, int, int]: ...
    def prune_static_deltas(
        self,
        commit: Optional[str] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def pull(
        self,
        remote_name: str,
        refs_to_fetch: Optional[Sequence[str]],
        flags: RepoPullFlags,
        progress: Optional[AsyncProgress] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    @staticmethod
    def pull_default_console_progress_changed(
        progress: AsyncProgress, user_data: None
    ) -> None: ...
    def pull_from_remotes_async(
        self,
        results: Sequence[RepoFinderResult],
        options: Optional[GLib.Variant] = None,
        progress: Optional[AsyncProgress] = None,
        cancellable: Optional[Gio.Cancellable] = None,
        callback: Optional[Callable[..., None]] = None,
        *user_data: Any,
    ) -> None: ...
    def pull_from_remotes_finish(self, result: Gio.AsyncResult) -> bool: ...
    def pull_one_dir(
        self,
        remote_name: str,
        dir_to_pull: str,
        refs_to_fetch: Optional[Sequence[str]],
        flags: RepoPullFlags,
        progress: Optional[AsyncProgress] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def pull_with_options(
        self,
        remote_name_or_baseurl: str,
        options: GLib.Variant,
        progress: Optional[AsyncProgress] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def query_object_storage_size(
        self,
        objtype: ObjectType,
        sha256: str,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, int]: ...
    def read_commit(
        self, ref: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, Gio.File, str]: ...
    def read_commit_detached_metadata(
        self, checksum: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, GLib.Variant]: ...
    def regenerate_metadata(
        self,
        additional_metadata: Optional[GLib.Variant] = None,
        options: Optional[GLib.Variant] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def regenerate_summary(
        self,
        additional_metadata: Optional[GLib.Variant] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def reload_config(self, cancellable: Optional[Gio.Cancellable] = None) -> bool: ...
    def remote_add(
        self,
        name: str,
        url: Optional[str] = None,
        options: Optional[GLib.Variant] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def remote_change(
        self,
        sysroot: Optional[Gio.File],
        changeop: RepoRemoteChange,
        name: str,
        url: Optional[str] = None,
        options: Optional[GLib.Variant] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def remote_delete(
        self, name: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...
    def remote_fetch_summary(
        self, name: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, GLib.Bytes, GLib.Bytes]: ...
    def remote_fetch_summary_with_options(
        self,
        name: str,
        options: Optional[GLib.Variant] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, GLib.Bytes, GLib.Bytes]: ...
    def remote_get_gpg_keys(
        self,
        name: Optional[str] = None,
        key_ids: Optional[Sequence[str]] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, list[GLib.Variant]]: ...
    def remote_get_gpg_verify(self, name: str) -> Tuple[bool, bool]: ...
    def remote_get_gpg_verify_summary(self, name: str) -> Tuple[bool, bool]: ...
    def remote_get_url(self, name: str) -> Tuple[bool, str]: ...
    def remote_gpg_import(
        self,
        name: str,
        source_stream: Optional[Gio.InputStream] = None,
        key_ids: Optional[Sequence[str]] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, int]: ...
    def remote_list(self) -> list[str]: ...
    def remote_list_collection_refs(
        self, remote_name: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, dict[CollectionRef, str]]: ...
    def remote_list_refs(
        self, remote_name: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, dict[str, str]]: ...
    def resolve_collection_ref(
        self,
        ref: CollectionRef,
        allow_noent: bool,
        flags: RepoResolveRevExtFlags,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, str]: ...
    def resolve_keyring_for_collection(
        self, collection_id: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> Remote: ...
    def resolve_rev(self, refspec: str, allow_noent: bool) -> Tuple[bool, str]: ...
    def resolve_rev_ext(
        self, refspec: str, allow_noent: bool, flags: RepoResolveRevExtFlags
    ) -> Tuple[bool, str]: ...
    def scan_hardlinks(self, cancellable: Optional[Gio.Cancellable] = None) -> bool: ...
    def set_alias_ref_immediate(
        self,
        remote: Optional[str],
        ref: str,
        target: Optional[str] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def set_cache_dir(
        self, dfd: int, path: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...
    def set_collection_id(self, collection_id: Optional[str] = None) -> bool: ...
    def set_collection_ref_immediate(
        self,
        ref: CollectionRef,
        checksum: Optional[str] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def set_disable_fsync(self, disable_fsync: bool) -> None: ...
    def set_ref_immediate(
        self,
        remote: Optional[str],
        ref: str,
        checksum: Optional[str] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def sign_commit(
        self,
        commit_checksum: str,
        key_id: str,
        homedir: Optional[str] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def sign_delta(
        self,
        from_commit: str,
        to_commit: str,
        key_id: str,
        homedir: str,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def signature_verify_commit_data(
        self,
        remote_name: str,
        commit_data: GLib.Bytes,
        commit_metadata: GLib.Bytes,
        flags: RepoVerifyFlags,
    ) -> Tuple[bool, str]: ...
    def static_delta_execute_offline(
        self,
        dir_or_file: Gio.File,
        skip_validation: bool,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def static_delta_execute_offline_with_signature(
        self,
        dir_or_file: Gio.File,
        sign: Sign,
        skip_validation: bool,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def static_delta_generate(
        self,
        opt: StaticDeltaGenerateOpt,
        from_: Optional[str],
        to: str,
        metadata: Optional[GLib.Variant] = None,
        params: Optional[GLib.Variant] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def static_delta_reindex(
        self,
        flags: StaticDeltaIndexFlags,
        opt_to_commit: str,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def static_delta_verify_signature(
        self, delta_id: str, sign: Sign
    ) -> Tuple[bool, str]: ...
    def transaction_set_collection_ref(
        self, ref: CollectionRef, checksum: Optional[str] = None
    ) -> None: ...
    def transaction_set_ref(
        self, remote: Optional[str], ref: str, checksum: Optional[str] = None
    ) -> None: ...
    def transaction_set_refspec(
        self, refspec: str, checksum: Optional[str] = None
    ) -> None: ...
    def traverse_commit(
        self,
        commit_checksum: str,
        maxdepth: int,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, dict[GLib.Variant, GLib.Variant]]: ...
    @staticmethod
    def traverse_new_parents() -> dict[GLib.Variant, GLib.Variant]: ...
    @staticmethod
    def traverse_new_reachable() -> dict[GLib.Variant, GLib.Variant]: ...
    @staticmethod
    def traverse_parents_get_commits(
        parents: dict[None, None], object: GLib.Variant
    ) -> list[str]: ...
    def traverse_reachable_refs(
        self,
        depth: int,
        reachable: dict[GLib.Variant, GLib.Variant],
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def verify_commit(
        self,
        commit_checksum: str,
        keyringdir: Optional[Gio.File] = None,
        extra_keyring: Optional[Gio.File] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def verify_commit_ext(
        self,
        commit_checksum: str,
        keyringdir: Optional[Gio.File] = None,
        extra_keyring: Optional[Gio.File] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> GpgVerifyResult: ...
    def verify_commit_for_remote(
        self,
        commit_checksum: str,
        remote_name: str,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> GpgVerifyResult: ...
    def verify_summary(
        self,
        remote_name: str,
        summary: GLib.Bytes,
        signatures: GLib.Bytes,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> GpgVerifyResult: ...
    def write_archive_to_mtree(
        self,
        archive: Gio.File,
        mtree: MutableTree,
        modifier: Optional[RepoCommitModifier],
        autocreate_parents: bool,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def write_archive_to_mtree_from_fd(
        self,
        fd: int,
        mtree: MutableTree,
        modifier: Optional[RepoCommitModifier],
        autocreate_parents: bool,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def write_commit(
        self,
        parent: Optional[str],
        subject: Optional[str],
        body: Optional[str],
        metadata: Optional[GLib.Variant],
        root: RepoFile,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, str]: ...
    def write_commit_detached_metadata(
        self,
        checksum: str,
        metadata: Optional[GLib.Variant] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def write_commit_with_time(
        self,
        parent: Optional[str],
        subject: Optional[str],
        body: Optional[str],
        metadata: Optional[GLib.Variant],
        root: RepoFile,
        time: int,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, str]: ...
    def write_config(self, new_config: GLib.KeyFile) -> bool: ...
    def write_content(
        self,
        expected_checksum: Optional[str],
        object_input: Gio.InputStream,
        length: int,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, bytes]: ...
    def write_content_async(
        self,
        expected_checksum: Optional[str],
        object: Gio.InputStream,
        length: int,
        cancellable: Optional[Gio.Cancellable] = None,
        callback: Optional[Callable[..., None]] = None,
        *user_data: Any,
    ) -> None: ...
    def write_content_finish(self, result: Gio.AsyncResult) -> Tuple[bool, int]: ...
    def write_content_trusted(
        self,
        checksum: str,
        object_input: Gio.InputStream,
        length: int,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def write_dfd_to_mtree(
        self,
        dfd: int,
        path: str,
        mtree: MutableTree,
        modifier: Optional[RepoCommitModifier] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def write_directory_to_mtree(
        self,
        dir: Gio.File,
        mtree: MutableTree,
        modifier: Optional[RepoCommitModifier] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def write_metadata(
        self,
        objtype: ObjectType,
        expected_checksum: Optional[str],
        object: GLib.Variant,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, bytes]: ...
    def write_metadata_async(
        self,
        objtype: ObjectType,
        expected_checksum: Optional[str],
        object: GLib.Variant,
        cancellable: Optional[Gio.Cancellable] = None,
        callback: Optional[Callable[..., None]] = None,
        *user_data: Any,
    ) -> None: ...
    def write_metadata_finish(self, result: Gio.AsyncResult) -> Tuple[bool, bytes]: ...
    def write_metadata_stream_trusted(
        self,
        objtype: ObjectType,
        checksum: str,
        object_input: Gio.InputStream,
        length: int,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def write_metadata_trusted(
        self,
        objtype: ObjectType,
        checksum: str,
        variant: GLib.Variant,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def write_mtree(
        self, mtree: MutableTree, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, Gio.File]: ...
    def write_regfile(
        self,
        expected_checksum: Optional[str],
        uid: int,
        gid: int,
        mode: int,
        content_len: int,
        xattrs: Optional[GLib.Variant] = None,
    ) -> ContentWriter: ...
    def write_regfile_inline(
        self,
        expected_checksum: Optional[str],
        uid: int,
        gid: int,
        mode: int,
        xattrs: Optional[GLib.Variant],
        buf: Sequence[int],
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> str: ...
    def write_symlink(
        self,
        expected_checksum: Optional[str],
        uid: int,
        gid: int,
        xattrs: Optional[GLib.Variant],
        symlink_target: str,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> str: ...

class RepoCheckoutAtOptions(GObject.GPointer):
    """
    :Constructors:

    ::

        RepoCheckoutAtOptions()
    """

    mode: RepoCheckoutMode = ...
    overwrite_mode: RepoCheckoutOverwriteMode = ...
    enable_uncompressed_cache: bool = ...
    enable_fsync: bool = ...
    process_whiteouts: bool = ...
    no_copy_fallback: bool = ...
    force_copy: bool = ...
    bareuseronly_dirs: bool = ...
    force_copy_zerosized: bool = ...
    process_passthrough_whiteouts: bool = ...
    unused_bools: list[bool] = ...
    subpath: str = ...
    devino_to_csum_cache: RepoDevInoCache = ...
    unused_ints: list[int] = ...
    unused_ptrs: list[None] = ...
    filter: Callable[..., RepoCheckoutFilterResult] = ...
    filter_user_data: None = ...
    sepolicy: SePolicy = ...
    sepolicy_prefix: str = ...
    def set_devino(self, cache: Optional[RepoDevInoCache] = None) -> None: ...

class RepoCommitModifier(GObject.GBoxed):
    """
    :Constructors:

    ::

        new(flags:OSTree.RepoCommitModifierFlags, commit_filter:OSTree.RepoCommitFilter=None, user_data=None) -> OSTree.RepoCommitModifier
    """

    @classmethod
    def new(
        cls,
        flags: RepoCommitModifierFlags,
        commit_filter: Optional[Callable[..., RepoCommitFilterResult]] = None,
        *user_data: Any,
    ) -> RepoCommitModifier: ...
    def ref(self) -> RepoCommitModifier: ...
    def set_devino_cache(self, cache: RepoDevInoCache) -> None: ...
    def set_sepolicy(self, sepolicy: Optional[SePolicy] = None) -> None: ...
    def set_sepolicy_from_commit(
        self, repo: Repo, rev: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...
    def set_xattr_callback(
        self, callback: Callable[..., GLib.Variant], *user_data: Any
    ) -> None: ...
    def unref(self) -> None: ...

class RepoCommitTraverseIter(GObject.GPointer):
    """
    :Constructors:

    ::

        RepoCommitTraverseIter()
    """

    initialized: bool = ...
    dummy: list[None] = ...
    dummy_checksum_data: list[int] = ...
    @staticmethod
    def cleanup(p: None) -> None: ...
    def clear(self) -> None: ...
    def get_dir(self) -> Tuple[str, str, str]: ...
    def get_file(self) -> Tuple[str, str]: ...
    def init_commit(
        self, repo: Repo, commit: GLib.Variant, flags: RepoCommitTraverseFlags
    ) -> bool: ...
    def init_dirtree(
        self, repo: Repo, dirtree: GLib.Variant, flags: RepoCommitTraverseFlags
    ) -> bool: ...
    def next(
        self, cancellable: Optional[Gio.Cancellable] = None
    ) -> RepoCommitIterResult: ...

class RepoDevInoCache(GObject.GBoxed):
    """
    :Constructors:

    ::

        new() -> OSTree.RepoDevInoCache
    """

    @classmethod
    def new(cls) -> RepoDevInoCache: ...
    def ref(self) -> RepoDevInoCache: ...
    def unref(self) -> None: ...

class RepoFile(GObject.Object, Gio.File):
    """
    :Constructors:

    ::

        RepoFile(**properties)

    Object OstreeRepoFile

    Signals from GObject:
      notify (GParam)
    """

    def ensure_resolved(self) -> bool: ...
    def get_checksum(self) -> str: ...
    def get_repo(self) -> Repo: ...
    def get_root(self) -> RepoFile: ...
    def get_xattrs(
        self, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, GLib.Variant]: ...
    def tree_find_child(self, name: str) -> Tuple[int, bool, GLib.Variant]: ...
    def tree_get_contents(self) -> Optional[GLib.Variant]: ...
    def tree_get_contents_checksum(self) -> Optional[str]: ...
    def tree_get_metadata(self) -> Optional[GLib.Variant]: ...
    def tree_get_metadata_checksum(self) -> Optional[str]: ...
    def tree_query_child(
        self,
        n: int,
        attributes: str,
        flags: Gio.FileQueryInfoFlags,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, Gio.FileInfo]: ...
    def tree_set_metadata(self, checksum: str, metadata: GLib.Variant) -> None: ...

class RepoFileClass(GObject.GPointer):
    """
    :Constructors:

    ::

        RepoFileClass()
    """

    parent_class: GObject.ObjectClass = ...

class RepoFinder(GObject.GInterface):
    """
    Interface OstreeRepoFinder

    Signals from GObject:
      notify (GParam)
    """

    @staticmethod
    def resolve_all_async(
        finders: Sequence[RepoFinder],
        refs: Sequence[CollectionRef],
        parent_repo: Repo,
        cancellable: Optional[Gio.Cancellable] = None,
        callback: Optional[Callable[..., None]] = None,
        *user_data: Any,
    ) -> None: ...
    @staticmethod
    def resolve_all_finish(result: Gio.AsyncResult) -> list[RepoFinderResult]: ...
    def resolve_async(
        self,
        refs: Sequence[CollectionRef],
        parent_repo: Repo,
        cancellable: Optional[Gio.Cancellable] = None,
        callback: Optional[Callable[..., None]] = None,
        *user_data: Any,
    ) -> None: ...
    def resolve_finish(self, result: Gio.AsyncResult) -> list[RepoFinderResult]: ...

class RepoFinderAvahi(GObject.Object, RepoFinder):
    """
    :Constructors:

    ::

        RepoFinderAvahi(**properties)
        new(context:GLib.MainContext=None) -> OSTree.RepoFinderAvahi

    Object OstreeRepoFinderAvahi

    Signals from GObject:
      notify (GParam)
    """

    @classmethod
    def new(cls, context: Optional[GLib.MainContext] = None) -> RepoFinderAvahi: ...
    def start(self) -> None: ...
    def stop(self) -> None: ...

class RepoFinderAvahiClass(GObject.GPointer):
    """
    :Constructors:

    ::

        RepoFinderAvahiClass()
    """

    parent_class: GObject.ObjectClass = ...

class RepoFinderConfig(GObject.Object, RepoFinder):
    """
    :Constructors:

    ::

        RepoFinderConfig(**properties)
        new() -> OSTree.RepoFinderConfig

    Object OstreeRepoFinderConfig

    Signals from GObject:
      notify (GParam)
    """

    @classmethod
    def new(cls) -> RepoFinderConfig: ...

class RepoFinderConfigClass(GObject.GPointer):
    """
    :Constructors:

    ::

        RepoFinderConfigClass()
    """

    parent_class: GObject.ObjectClass = ...

class RepoFinderInterface(GObject.GPointer):
    """
    :Constructors:

    ::

        RepoFinderInterface()
    """

    g_iface: GObject.TypeInterface = ...
    resolve_async: Callable[..., None] = ...
    resolve_finish: Callable[
        [RepoFinder, Gio.AsyncResult], list[RepoFinderResult]
    ] = ...

class RepoFinderMount(GObject.Object, RepoFinder):
    """
    :Constructors:

    ::

        RepoFinderMount(**properties)
        new(monitor:Gio.VolumeMonitor=None) -> OSTree.RepoFinderMount

    Object OstreeRepoFinderMount

    Properties from OstreeRepoFinderMount:
      monitor -> GVolumeMonitor: Volume Monitor
        Volume monitor to use to look up mounted volumes when queried.

    Signals from GObject:
      notify (GParam)
    """

    class Props:
        monitor: Gio.VolumeMonitor

    props: Props = ...
    def __init__(self, monitor: Gio.VolumeMonitor = ...): ...
    @classmethod
    def new(cls, monitor: Optional[Gio.VolumeMonitor] = None) -> RepoFinderMount: ...

class RepoFinderMountClass(GObject.GPointer):
    """
    :Constructors:

    ::

        RepoFinderMountClass()
    """

    parent_class: GObject.ObjectClass = ...

class RepoFinderOverride(GObject.Object, RepoFinder):
    """
    :Constructors:

    ::

        RepoFinderOverride(**properties)
        new() -> OSTree.RepoFinderOverride

    Object OstreeRepoFinderOverride

    Signals from GObject:
      notify (GParam)
    """

    def add_uri(self, uri: str) -> None: ...
    @classmethod
    def new(cls) -> RepoFinderOverride: ...

class RepoFinderOverrideClass(GObject.GPointer):
    """
    :Constructors:

    ::

        RepoFinderOverrideClass()
    """

    parent_class: GObject.ObjectClass = ...

class RepoFinderResult(GObject.GBoxed):
    """
    :Constructors:

    ::

        RepoFinderResult()
        new(remote:OSTree.Remote, finder:OSTree.RepoFinder, priority:int, ref_to_checksum:dict, ref_to_timestamp:dict=None, summary_last_modified:int) -> OSTree.RepoFinderResult
    """

    remote: Remote = ...
    finder: RepoFinder = ...
    priority: int = ...
    ref_to_checksum: dict[CollectionRef, str] = ...
    summary_last_modified: int = ...
    ref_to_timestamp: dict[CollectionRef, int] = ...
    padding: list[None] = ...
    def compare(self, b: RepoFinderResult) -> int: ...
    def dup(self) -> RepoFinderResult: ...
    def free(self) -> None: ...
    @staticmethod
    def freev(results: Sequence[RepoFinderResult]) -> None: ...
    @classmethod
    def new(
        cls,
        remote: Remote,
        finder: RepoFinder,
        priority: int,
        ref_to_checksum: dict[CollectionRef, str],
        ref_to_timestamp: Optional[dict[CollectionRef, int]],
        summary_last_modified: int,
    ) -> RepoFinderResult: ...

class RepoPruneOptions(GObject.GPointer):
    """
    :Constructors:

    ::

        RepoPruneOptions()
    """

    flags: RepoPruneFlags = ...
    reachable: dict[None, None] = ...
    unused_bools: list[bool] = ...
    unused_ints: list[int] = ...
    unused_ptrs: list[None] = ...

class RepoTransactionStats(GObject.GBoxed):
    """
    :Constructors:

    ::

        RepoTransactionStats()
    """

    metadata_objects_total: int = ...
    metadata_objects_written: int = ...
    content_objects_total: int = ...
    content_objects_written: int = ...
    content_bytes_written: int = ...
    devino_cache_hits: int = ...
    padding1: int = ...
    padding2: int = ...
    padding3: int = ...
    padding4: int = ...

class SePolicy(GObject.Object, Gio.Initable):
    """
    :Constructors:

    ::

        SePolicy(**properties)
        new(path:Gio.File, cancellable:Gio.Cancellable=None) -> OSTree.SePolicy
        new_at(rootfs_dfd:int, cancellable:Gio.Cancellable=None) -> OSTree.SePolicy
        new_from_commit(repo:OSTree.Repo, rev:str, cancellable:Gio.Cancellable=None) -> OSTree.SePolicy

    Object OstreeSePolicy

    Properties from OstreeSePolicy:
      path -> GFile:

      rootfs-dfd -> gint:


    Signals from GObject:
      notify (GParam)
    """

    class Props:
        path: Optional[Gio.File]
        rootfs_dfd: int

    props: Props = ...
    def __init__(self, path: Gio.File = ..., rootfs_dfd: int = ...): ...
    @staticmethod
    def fscreatecon_cleanup(unused: None) -> None: ...
    def get_csum(self) -> Optional[str]: ...
    def get_label(
        self,
        relpath: str,
        unix_mode: int,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, str]: ...
    def get_name(self) -> str: ...
    def get_path(self) -> Optional[Gio.File]: ...
    @classmethod
    def new(
        cls, path: Gio.File, cancellable: Optional[Gio.Cancellable] = None
    ) -> SePolicy: ...
    @classmethod
    def new_at(
        cls, rootfs_dfd: int, cancellable: Optional[Gio.Cancellable] = None
    ) -> SePolicy: ...
    @classmethod
    def new_from_commit(
        cls, repo: Repo, rev: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> SePolicy: ...
    def restorecon(
        self,
        path: str,
        info: Optional[Gio.FileInfo],
        target: Gio.File,
        flags: SePolicyRestoreconFlags,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, str]: ...
    def setfscreatecon(self, path: str, mode: int) -> bool: ...

class Sign(GObject.GInterface):
    """
    Interface OstreeSign

    Signals from GObject:
      notify (GParam)
    """

    def add_pk(self, public_key: GLib.Variant) -> bool: ...
    def clear_keys(self) -> bool: ...
    def commit(
        self,
        repo: Repo,
        commit_checksum: str,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def commit_verify(
        self,
        repo: Repo,
        commit_checksum: str,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, str]: ...
    def data(
        self, data: GLib.Bytes, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, GLib.Bytes]: ...
    def data_verify(
        self, data: GLib.Bytes, signatures: GLib.Variant
    ) -> Tuple[bool, str]: ...
    def ed25519_add_pk(self, public_key: GLib.Variant) -> bool: ...
    def ed25519_clear_keys(self) -> bool: ...
    def ed25519_data(
        self,
        data: GLib.Bytes,
        signature: GLib.Bytes,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def ed25519_data_verify(
        self, data: GLib.Bytes, signatures: GLib.Variant, out_success_message: str
    ) -> bool: ...
    def ed25519_get_name(self) -> str: ...
    def ed25519_load_pk(self, options: GLib.Variant) -> bool: ...
    def ed25519_metadata_format(self) -> str: ...
    def ed25519_metadata_key(self) -> str: ...
    def ed25519_set_pk(self, public_key: GLib.Variant) -> bool: ...
    def ed25519_set_sk(self, secret_key: GLib.Variant) -> bool: ...
    @staticmethod
    def get_all() -> list[Sign]: ...
    @staticmethod
    def get_by_name(name: str) -> Sign: ...
    def get_name(self) -> str: ...
    def load_pk(self, options: GLib.Variant) -> bool: ...
    def metadata_format(self) -> str: ...
    def metadata_key(self) -> str: ...
    def set_pk(self, public_key: GLib.Variant) -> bool: ...
    def set_sk(self, secret_key: GLib.Variant) -> bool: ...
    def summary(
        self,
        repo: Repo,
        keys: GLib.Variant,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...

class SignEd25519(GObject.GPointer): ...

class SignEd25519Class(GObject.GPointer):
    """
    :Constructors:

    ::

        SignEd25519Class()
    """

    parent_class: GObject.ObjectClass = ...

class SignInterface(GObject.GPointer):
    """
    :Constructors:

    ::

        SignInterface()
    """

    g_iface: GObject.TypeInterface = ...
    get_name: Callable[[Sign], str] = ...
    data: Callable[
        [Sign, GLib.Bytes, Optional[Gio.Cancellable]], Tuple[bool, GLib.Bytes]
    ] = ...
    data_verify: Callable[[Sign, GLib.Bytes, GLib.Variant], Tuple[bool, str]] = ...
    metadata_key: Callable[[Sign], str] = ...
    metadata_format: Callable[[Sign], str] = ...
    clear_keys: Callable[[Sign], bool] = ...
    set_sk: Callable[[Sign, GLib.Variant], bool] = ...
    set_pk: Callable[[Sign, GLib.Variant], bool] = ...
    add_pk: Callable[[Sign, GLib.Variant], bool] = ...
    load_pk: Callable[[Sign, GLib.Variant], bool] = ...

class Sysroot(GObject.Object):
    """
    :Constructors:

    ::

        Sysroot(**properties)
        new(path:Gio.File=None) -> OSTree.Sysroot
        new_default() -> OSTree.Sysroot

    Object OstreeSysroot

    Signals from OstreeSysroot:
      journal-msg (gchararray)

    Properties from OstreeSysroot:
      path -> GFile:


    Signals from GObject:
      notify (GParam)
    """

    class Props:
        path: Gio.File

    props: Props = ...
    def __init__(self, path: Gio.File = ...): ...
    def cleanup(self, cancellable: Optional[Gio.Cancellable] = None) -> bool: ...
    def cleanup_prune_repo(
        self, options: RepoPruneOptions, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, int, int, int]: ...
    def deploy_tree(
        self,
        osname: Optional[str],
        revision: str,
        origin: Optional[GLib.KeyFile] = None,
        provided_merge_deployment: Optional[Deployment] = None,
        override_kernel_argv: Optional[Sequence[str]] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, Deployment]: ...
    def deploy_tree_with_options(
        self,
        osname: Optional[str],
        revision: str,
        origin: Optional[GLib.KeyFile] = None,
        provided_merge_deployment: Optional[Deployment] = None,
        opts: Optional[SysrootDeployTreeOpts] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, Deployment]: ...
    def deployment_set_kargs(
        self,
        deployment: Deployment,
        new_kargs: Sequence[str],
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def deployment_set_kargs_in_place(
        self,
        deployment: Deployment,
        kargs_str: Optional[str] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def deployment_set_mutable(
        self,
        deployment: Deployment,
        is_mutable: bool,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def deployment_set_pinned(
        self, deployment: Deployment, is_pinned: bool
    ) -> bool: ...
    def deployment_unlock(
        self,
        deployment: Deployment,
        unlocked_state: DeploymentUnlockedState,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def ensure_initialized(
        self, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...
    def get_booted_deployment(self) -> Optional[Deployment]: ...
    def get_bootversion(self) -> int: ...
    def get_deployment_directory(self, deployment: Deployment) -> Gio.File: ...
    def get_deployment_dirpath(self, deployment: Deployment) -> str: ...
    @staticmethod
    def get_deployment_origin_path(deployment_path: Gio.File) -> Gio.File: ...
    def get_deployments(self) -> list[Deployment]: ...
    def get_fd(self) -> int: ...
    def get_merge_deployment(
        self, osname: Optional[str] = None
    ) -> Optional[Deployment]: ...
    def get_path(self) -> Gio.File: ...
    def get_repo(
        self, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, Repo]: ...
    def get_staged_deployment(self) -> Optional[Deployment]: ...
    def get_subbootversion(self) -> int: ...
    def init_osname(
        self, osname: str, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...
    def initialize(self) -> bool: ...
    def initialize_with_mount_namespace(
        self, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...
    def is_booted(self) -> bool: ...
    def load(self, cancellable: Optional[Gio.Cancellable] = None) -> bool: ...
    def load_if_changed(
        self, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, bool]: ...
    def lock(self) -> bool: ...
    def lock_async(
        self,
        cancellable: Optional[Gio.Cancellable] = None,
        callback: Optional[Callable[..., None]] = None,
        *user_data: Any,
    ) -> None: ...
    def lock_finish(self, result: Gio.AsyncResult) -> bool: ...
    @classmethod
    def new(cls, path: Optional[Gio.File] = None) -> Sysroot: ...
    @classmethod
    def new_default(cls) -> Sysroot: ...
    def origin_new_from_refspec(self, refspec: str) -> GLib.KeyFile: ...
    def prepare_cleanup(
        self, cancellable: Optional[Gio.Cancellable] = None
    ) -> bool: ...
    def query_deployments_for(
        self, osname: Optional[str] = None
    ) -> Tuple[Deployment, Deployment]: ...
    def repo(self) -> Repo: ...
    def require_booted_deployment(self) -> Deployment: ...
    def set_mount_namespace_in_use(self) -> None: ...
    def simple_write_deployment(
        self,
        osname: Optional[str],
        new_deployment: Deployment,
        merge_deployment: Optional[Deployment],
        flags: SysrootSimpleWriteDeploymentFlags,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def stage_overlay_initrd(
        self, fd: int, cancellable: Optional[Gio.Cancellable] = None
    ) -> Tuple[bool, str]: ...
    def stage_tree(
        self,
        osname: Optional[str],
        revision: str,
        origin: Optional[GLib.KeyFile] = None,
        merge_deployment: Optional[Deployment] = None,
        override_kernel_argv: Optional[Sequence[str]] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, Deployment]: ...
    def stage_tree_with_options(
        self,
        osname: Optional[str],
        revision: str,
        origin: Optional[GLib.KeyFile],
        merge_deployment: Optional[Deployment],
        opts: SysrootDeployTreeOpts,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, Deployment]: ...
    def try_lock(self) -> Tuple[bool, bool]: ...
    def unload(self) -> None: ...
    def unlock(self) -> None: ...
    def write_deployments(
        self,
        new_deployments: Sequence[Deployment],
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def write_deployments_with_options(
        self,
        new_deployments: Sequence[Deployment],
        opts: SysrootWriteDeploymentsOpts,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...
    def write_origin_file(
        self,
        deployment: Deployment,
        new_origin: Optional[GLib.KeyFile] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...

class SysrootDeployTreeOpts(GObject.GPointer):
    """
    :Constructors:

    ::

        SysrootDeployTreeOpts()
    """

    unused_bools: list[bool] = ...
    unused_ints: list[int] = ...
    override_kernel_argv: str = ...
    overlay_initrds: str = ...
    unused_ptrs: list[None] = ...

class SysrootUpgrader(GObject.Object, Gio.Initable):
    """
    :Constructors:

    ::

        SysrootUpgrader(**properties)
        new(sysroot:OSTree.Sysroot, cancellable:Gio.Cancellable=None) -> OSTree.SysrootUpgrader
        new_for_os(sysroot:OSTree.Sysroot, osname:str=None, cancellable:Gio.Cancellable=None) -> OSTree.SysrootUpgrader
        new_for_os_with_flags(sysroot:OSTree.Sysroot, osname:str=None, flags:OSTree.SysrootUpgraderFlags, cancellable:Gio.Cancellable=None) -> OSTree.SysrootUpgrader

    Object OstreeSysrootUpgrader

    Properties from OstreeSysrootUpgrader:
      sysroot -> OstreeSysroot:

      osname -> gchararray:

      flags -> OstreeSysrootUpgraderFlags:


    Signals from GObject:
      notify (GParam)
    """

    class Props:
        flags: SysrootUpgraderFlags
        osname: str
        sysroot: Sysroot

    props: Props = ...
    def __init__(
        self,
        flags: SysrootUpgraderFlags = ...,
        osname: str = ...,
        sysroot: Sysroot = ...,
    ): ...
    @staticmethod
    def check_timestamps(repo: Repo, from_rev: str, to_rev: str) -> bool: ...
    def deploy(self, cancellable: Optional[Gio.Cancellable] = None) -> bool: ...
    def dup_origin(self) -> Optional[GLib.KeyFile]: ...
    def get_origin(self) -> Optional[GLib.KeyFile]: ...
    def get_origin_description(self) -> Optional[str]: ...
    @classmethod
    def new(
        cls, sysroot: Sysroot, cancellable: Optional[Gio.Cancellable] = None
    ) -> SysrootUpgrader: ...
    @classmethod
    def new_for_os(
        cls,
        sysroot: Sysroot,
        osname: Optional[str] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> SysrootUpgrader: ...
    @classmethod
    def new_for_os_with_flags(
        cls,
        sysroot: Sysroot,
        osname: Optional[str],
        flags: SysrootUpgraderFlags,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> SysrootUpgrader: ...
    def pull(
        self,
        flags: RepoPullFlags,
        upgrader_flags: SysrootUpgraderPullFlags,
        progress: Optional[AsyncProgress] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, bool]: ...
    def pull_one_dir(
        self,
        dir_to_pull: str,
        flags: RepoPullFlags,
        upgrader_flags: SysrootUpgraderPullFlags,
        progress: Optional[AsyncProgress] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> Tuple[bool, bool]: ...
    def set_origin(
        self,
        origin: Optional[GLib.KeyFile] = None,
        cancellable: Optional[Gio.Cancellable] = None,
    ) -> bool: ...

class SysrootWriteDeploymentsOpts(GObject.GPointer):
    """
    :Constructors:

    ::

        SysrootWriteDeploymentsOpts()
    """

    do_postclean: bool = ...
    disable_auto_early_prune: bool = ...
    unused_bools: list[bool] = ...
    unused_ints: list[int] = ...
    unused_ptrs: list[None] = ...

class ChecksumFlags(GObject.GFlags):
    CANONICAL_PERMISSIONS = 2
    IGNORE_XATTRS = 1
    NONE = 0

class DiffFlags(GObject.GFlags):
    IGNORE_XATTRS = 1
    NONE = 0

class GpgSignatureFormatFlags(GObject.GFlags):
    GPG_SIGNATURE_FORMAT_DEFAULT = 0

class RepoCommitModifierFlags(GObject.GFlags):
    CANONICAL_PERMISSIONS = 4
    CONSUME = 16
    DEVINO_CANONICAL = 32
    ERROR_ON_UNLABELED = 8
    GENERATE_SIZES = 2
    NONE = 0
    SKIP_XATTRS = 1

class RepoCommitState(GObject.GFlags):
    FSCK_PARTIAL = 2
    NORMAL = 0
    PARTIAL = 1

class RepoCommitTraverseFlags(GObject.GFlags):
    COMMIT_ONLY = 2
    NONE = 1

class RepoListObjectsFlags(GObject.GFlags):
    ALL = 4
    LOOSE = 1
    NO_PARENTS = 8
    PACKED = 2

class RepoListRefsExtFlags(GObject.GFlags):
    ALIASES = 1
    EXCLUDE_MIRRORS = 4
    EXCLUDE_REMOTES = 2
    NONE = 0

class RepoPruneFlags(GObject.GFlags):
    COMMIT_ONLY = 4
    NONE = 0
    NO_PRUNE = 1
    REFS_ONLY = 2

class RepoPullFlags(GObject.GFlags):
    BAREUSERONLY_FILES = 8
    COMMIT_ONLY = 2
    MIRROR = 1
    NONE = 0
    TRUSTED_HTTP = 16
    UNTRUSTED = 4

class RepoResolveRevExtFlags(GObject.GFlags):
    LOCAL_ONLY = 1
    NONE = 0

class RepoVerifyFlags(GObject.GFlags):
    NONE = 0
    NO_GPG = 1
    NO_SIGNAPI = 2

class SePolicyRestoreconFlags(GObject.GFlags):
    ALLOW_NOLABEL = 1
    KEEP_EXISTING = 2
    NONE = 0

class SysrootSimpleWriteDeploymentFlags(GObject.GFlags):
    NONE = 0
    NOT_DEFAULT = 2
    NO_CLEAN = 4
    RETAIN = 1
    RETAIN_PENDING = 8
    RETAIN_ROLLBACK = 16

class SysrootUpgraderFlags(GObject.GFlags):
    IGNORE_UNCONFIGURED = 2
    STAGE = 4

class SysrootUpgraderPullFlags(GObject.GFlags):
    ALLOW_OLDER = 1
    NONE = 0
    SYNTHETIC = 2

class DeploymentUnlockedState(GObject.GEnum):
    DEVELOPMENT = 1
    HOTFIX = 2
    NONE = 0
    TRANSIENT = 3

class GpgError(GObject.GEnum):
    EXPIRED_KEY = 4
    EXPIRED_SIGNATURE = 3
    INVALID_SIGNATURE = 1
    MISSING_KEY = 2
    NO_SIGNATURE = 0
    REVOKED_KEY = 5

class GpgSignatureAttr(GObject.GEnum):
    EXP_TIMESTAMP = 7
    FINGERPRINT = 5
    FINGERPRINT_PRIMARY = 12
    HASH_ALGO_NAME = 9
    KEY_EXPIRED = 2
    KEY_EXP_TIMESTAMP = 13
    KEY_EXP_TIMESTAMP_PRIMARY = 14
    KEY_MISSING = 4
    KEY_REVOKED = 3
    PUBKEY_ALGO_NAME = 8
    SIG_EXPIRED = 1
    TIMESTAMP = 6
    USER_EMAIL = 11
    USER_NAME = 10
    VALID = 0

class ObjectType(GObject.GEnum):
    COMMIT = 4
    COMMIT_META = 6
    DIR_META = 3
    DIR_TREE = 2
    FILE = 1
    FILE_XATTRS = 8
    FILE_XATTRS_LINK = 9
    PAYLOAD_LINK = 7
    TOMBSTONE_COMMIT = 5

class RepoCheckoutFilterResult(GObject.GEnum):
    ALLOW = 0
    SKIP = 1

class RepoCheckoutMode(GObject.GEnum):
    NONE = 0
    USER = 1

class RepoCheckoutOverwriteMode(GObject.GEnum):
    ADD_FILES = 2
    NONE = 0
    UNION_FILES = 1
    UNION_IDENTICAL = 3

class RepoCommitFilterResult(GObject.GEnum):
    ALLOW = 0
    SKIP = 1

class RepoCommitIterResult(GObject.GEnum):
    DIR = 3
    END = 1
    ERROR = 0
    FILE = 2

class RepoLockType(GObject.GEnum):
    EXCLUSIVE = 1
    SHARED = 0

class RepoMode(GObject.GEnum):
    ARCHIVE = 1
    ARCHIVE_Z2 = 1
    BARE = 0
    BARE_SPLIT_XATTRS = 4
    BARE_USER = 2
    BARE_USER_ONLY = 3

class RepoRemoteChange(GObject.GEnum):
    ADD = 0
    ADD_IF_NOT_EXISTS = 1
    DELETE = 2
    DELETE_IF_EXISTS = 3
    REPLACE = 4

class StaticDeltaGenerateOpt(GObject.GEnum):
    LOWLATENCY = 0
    MAJOR = 1

class StaticDeltaIndexFlags(GObject.GEnum):
    STATIC_DELTA_INDEX_FLAGS_NONE = 0
